"""Array-size annotations for OpenGL.raw.GLE

Automatically generated by the generateraw script, do not edit!
"""
from OpenGL.raw import GLE as raw

from ctypes import *
from OpenGL import platform, arrays
from OpenGL.constant import Constant
from OpenGL import constants as GLconstants
GLvoid = GLconstants.GLvoid

STRING = c_char_p


gleExtrusion = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		arrays.setInputArraySizeType(
			arrays.setInputArraySizeType(
				arrays.setInputArraySizeType(
					raw.gleExtrusion,
					None, # XXX Could not determine size of argument contour for gleExtrusion arrays.GLdoubleArray
					arrays.GLdoubleArray, 
					'contour',
				),
				None, # XXX Could not determine size of argument cont_normal for gleExtrusion arrays.GLdoubleArray
				arrays.GLdoubleArray, 
				'cont_normal',
			),
			None, # XXX Could not determine size of argument up for gleExtrusion arrays.GLdoubleArray
			arrays.GLdoubleArray, 
			'up',
		),
		None, # XXX Could not determine size of argument point_array for gleExtrusion arrays.GLdoubleArray
		arrays.GLdoubleArray, 
		'point_array',
	),
	None, # XXX Could not determine size of argument color_array for gleExtrusion arrays.GLfloatArray
	arrays.GLfloatArray, 
	'color_array',
)

gleHelicoid = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		raw.gleHelicoid,
		None, # XXX Could not determine size of argument startXform for gleHelicoid arrays.GLdoubleArray
		arrays.GLdoubleArray, 
		'startXform',
	),
	None, # XXX Could not determine size of argument dXformdTheta for gleHelicoid arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'dXformdTheta',
)

gleLathe = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		arrays.setInputArraySizeType(
			arrays.setInputArraySizeType(
				arrays.setInputArraySizeType(
					raw.gleLathe,
					None, # XXX Could not determine size of argument contour for gleLathe arrays.GLdoubleArray
					arrays.GLdoubleArray, 
					'contour',
				),
				None, # XXX Could not determine size of argument cont_normal for gleLathe arrays.GLdoubleArray
				arrays.GLdoubleArray, 
				'cont_normal',
			),
			None, # XXX Could not determine size of argument up for gleLathe arrays.GLdoubleArray
			arrays.GLdoubleArray, 
			'up',
		),
		None, # XXX Could not determine size of argument startXform for gleLathe arrays.GLdoubleArray
		arrays.GLdoubleArray, 
		'startXform',
	),
	None, # XXX Could not determine size of argument dXformdTheta for gleLathe arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'dXformdTheta',
)

glePolyCone = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		arrays.setInputArraySizeType(
			raw.glePolyCone,
			None, # XXX Could not determine size of argument point_array for glePolyCone arrays.GLdoubleArray
			arrays.GLdoubleArray, 
			'point_array',
		),
		None, # XXX Could not determine size of argument color_array for glePolyCone arrays.GLfloatArray
		arrays.GLfloatArray, 
		'color_array',
	),
	None, # XXX Could not determine size of argument radius_array for glePolyCone arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'radius_array',
)

glePolyCylinder = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		raw.glePolyCylinder,
		None, # XXX Could not determine size of argument point_array for glePolyCylinder arrays.GLdoubleArray
		arrays.GLdoubleArray, 
		'point_array',
	),
	None, # XXX Could not determine size of argument color_array for glePolyCylinder arrays.GLfloatArray
	arrays.GLfloatArray, 
	'color_array',
)

gleScrew = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		arrays.setInputArraySizeType(
			raw.gleScrew,
			None, # XXX Could not determine size of argument contour for gleScrew arrays.GLdoubleArray
			arrays.GLdoubleArray, 
			'contour',
		),
		None, # XXX Could not determine size of argument cont_normal for gleScrew arrays.GLdoubleArray
		arrays.GLdoubleArray, 
		'cont_normal',
	),
	None, # XXX Could not determine size of argument up for gleScrew arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'up',
)

gleSpiral = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		arrays.setInputArraySizeType(
			arrays.setInputArraySizeType(
				arrays.setInputArraySizeType(
					raw.gleSpiral,
					None, # XXX Could not determine size of argument contour for gleSpiral arrays.GLdoubleArray
					arrays.GLdoubleArray, 
					'contour',
				),
				None, # XXX Could not determine size of argument cont_normal for gleSpiral arrays.GLdoubleArray
				arrays.GLdoubleArray, 
				'cont_normal',
			),
			None, # XXX Could not determine size of argument up for gleSpiral arrays.GLdoubleArray
			arrays.GLdoubleArray, 
			'up',
		),
		None, # XXX Could not determine size of argument startXform for gleSpiral arrays.GLdoubleArray
		arrays.GLdoubleArray, 
		'startXform',
	),
	None, # XXX Could not determine size of argument dXformdTheta for gleSpiral arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'dXformdTheta',
)

gleSuperExtrusion = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		arrays.setInputArraySizeType(
			arrays.setInputArraySizeType(
				arrays.setInputArraySizeType(
					arrays.setInputArraySizeType(
						raw.gleSuperExtrusion,
						None, # XXX Could not determine size of argument contour for gleSuperExtrusion arrays.GLdoubleArray
						arrays.GLdoubleArray, 
						'contour',
					),
					None, # XXX Could not determine size of argument cont_normal for gleSuperExtrusion arrays.GLdoubleArray
					arrays.GLdoubleArray, 
					'cont_normal',
				),
				None, # XXX Could not determine size of argument up for gleSuperExtrusion arrays.GLdoubleArray
				arrays.GLdoubleArray, 
				'up',
			),
			None, # XXX Could not determine size of argument point_array for gleSuperExtrusion arrays.GLdoubleArray
			arrays.GLdoubleArray, 
			'point_array',
		),
		None, # XXX Could not determine size of argument color_array for gleSuperExtrusion arrays.GLfloatArray
		arrays.GLfloatArray, 
		'color_array',
	),
	None, # XXX Could not determine size of argument xform_array for gleSuperExtrusion arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'xform_array',
)

gleToroid = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		raw.gleToroid,
		None, # XXX Could not determine size of argument startXform for gleToroid arrays.GLdoubleArray
		arrays.GLdoubleArray, 
		'startXform',
	),
	None, # XXX Could not determine size of argument dXformdTheta for gleToroid arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'dXformdTheta',
)

gleTwistExtrusion = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		arrays.setInputArraySizeType(
			arrays.setInputArraySizeType(
				arrays.setInputArraySizeType(
					arrays.setInputArraySizeType(
						raw.gleTwistExtrusion,
						None, # XXX Could not determine size of argument contour for gleTwistExtrusion arrays.GLdoubleArray
						arrays.GLdoubleArray, 
						'contour',
					),
					None, # XXX Could not determine size of argument cont_normal for gleTwistExtrusion arrays.GLdoubleArray
					arrays.GLdoubleArray, 
					'cont_normal',
				),
				None, # XXX Could not determine size of argument up for gleTwistExtrusion arrays.GLdoubleArray
				arrays.GLdoubleArray, 
				'up',
			),
			None, # XXX Could not determine size of argument point_array for gleTwistExtrusion arrays.GLdoubleArray
			arrays.GLdoubleArray, 
			'point_array',
		),
		None, # XXX Could not determine size of argument color_array for gleTwistExtrusion arrays.GLfloatArray
		arrays.GLfloatArray, 
		'color_array',
	),
	None, # XXX Could not determine size of argument twist_array for gleTwistExtrusion arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'twist_array',
)

rot_about_axis = arrays.setInputArraySizeType(
	raw.rot_about_axis,
	None, # XXX Could not determine size of argument axis for rot_about_axis arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'axis',
)

rot_axis = arrays.setInputArraySizeType(
	raw.rot_axis,
	None, # XXX Could not determine size of argument axis for rot_axis arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'axis',
)

rot_omega = arrays.setInputArraySizeType(
	raw.rot_omega,
	None, # XXX Could not determine size of argument axis for rot_omega arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'axis',
)

urot_about_axis = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		raw.urot_about_axis,
		None, # XXX Could not determine size of argument m for urot_about_axis arrays.GLdoubleArray
		arrays.GLdoubleArray, 
		'm',
	),
	None, # XXX Could not determine size of argument axis for urot_about_axis arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'axis',
)

urot_axis = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		raw.urot_axis,
		None, # XXX Could not determine size of argument m for urot_axis arrays.GLdoubleArray
		arrays.GLdoubleArray, 
		'm',
	),
	None, # XXX Could not determine size of argument axis for urot_axis arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'axis',
)

urot_omega = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		raw.urot_omega,
		None, # XXX Could not determine size of argument m for urot_omega arrays.GLdoubleArray
		arrays.GLdoubleArray, 
		'm',
	),
	None, # XXX Could not determine size of argument axis for urot_omega arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'axis',
)

urot_prince = arrays.setInputArraySizeType(
	raw.urot_prince,
	None, # XXX Could not determine size of argument m for urot_prince arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'm',
)

uview_direction = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		arrays.setInputArraySizeType(
			raw.uview_direction,
			None, # XXX Could not determine size of argument m for uview_direction arrays.GLdoubleArray
			arrays.GLdoubleArray, 
			'm',
		),
		None, # XXX Could not determine size of argument v21 for uview_direction arrays.GLdoubleArray
		arrays.GLdoubleArray, 
		'v21',
	),
	None, # XXX Could not determine size of argument up for uview_direction arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'up',
)

uviewpoint = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		arrays.setInputArraySizeType(
			arrays.setInputArraySizeType(
				raw.uviewpoint,
				None, # XXX Could not determine size of argument m for uviewpoint arrays.GLdoubleArray
				arrays.GLdoubleArray, 
				'm',
			),
			None, # XXX Could not determine size of argument v1 for uviewpoint arrays.GLdoubleArray
			arrays.GLdoubleArray, 
			'v1',
		),
		None, # XXX Could not determine size of argument v2 for uviewpoint arrays.GLdoubleArray
		arrays.GLdoubleArray, 
		'v2',
	),
	None, # XXX Could not determine size of argument up for uviewpoint arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'up',
)

__all__ = [
	'gleExtrusion',
	'gleHelicoid',
	'gleLathe',
	'glePolyCone',
	'glePolyCylinder',
	'gleScrew',
	'gleSpiral',
	'gleSuperExtrusion',
	'gleToroid',
	'gleTwistExtrusion',
	'rot_about_axis',
	'rot_axis',
	'rot_omega',
	'urot_about_axis',
	'urot_axis',
	'urot_omega',
	'urot_prince',
	'uview_direction',
	'uviewpoint'
]




'''OpenGL extension NV.light_max_exponent

Overview (from the spec)
	
	Default OpenGL does not permit a shininess or spot exponent over
	128.0.  This extension permits implementations to support and
	advertise a maximum shininess and spot exponent beyond 128.0.
	
	Note that extremely high exponents for shininess and/or spot light
	cutoff will require sufficiently high tessellation for acceptable
	lighting results.
	
	Paul Deifenbach's thesis suggests that higher exponents are
	necessary to approximate BRDFs with per-vertex ligthing and
	multiple passes.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/light_max_exponent.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_light_max_exponent'
GL_MAX_SHININESS_NV = constant.Constant( 'GL_MAX_SHININESS_NV', 0x8504 )
glget.addGLGetConstant( GL_MAX_SHININESS_NV, (1,) )
GL_MAX_SPOT_EXPONENT_NV = constant.Constant( 'GL_MAX_SPOT_EXPONENT_NV', 0x8505 )
glget.addGLGetConstant( GL_MAX_SPOT_EXPONENT_NV, (1,) )


def glInitLightMaxExponentNV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Module to provide numeric-module name for current version"""
def moduleName( module ):
	"""Given a numpy, numeric or numarray module, calculate helper module name"""
	name = module.__name__.lower()
	version = '_'.join( module.__version__.split('.'))
	return 'OpenGL.arrays._%(name)s_%(version)s'%locals()
def loadModule( name ):
	"""Load the current C module for our numeric version"""
	return __import__( name, {}, {}, name.split( '.' ))



'''OpenGL extension EXT.subtexture

Overview (from the spec)
	
	This extension allows a contiguous portion of an already-existing
	texture image to be redefined, without affecting the remaining portion
	of the image, or any of the other state that describe the texture.  No
	provision is made to query a subregion of a texture.
	
	Semantics for null image pointers are defined for TexImage1D,
	TexImage2D, and TexImage3DEXT.  Null image pointers can be used by
	applications to effectively support texture arrays whose dimensions
	are not a power of 2.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/subtexture.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_subtexture'

glTexSubImage1DEXT = platform.createExtensionFunction( 
	'glTexSubImage1DEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint, constants.GLint, constants.GLsizei, constants.GLenum, constants.GLenum, ctypes.c_void_p,),
	doc = 'glTexSubImage1DEXT( GLenum(target), GLint(level), GLint(xoffset), GLsizei(width), GLenum(format), GLenum(type), c_void_p(pixels) ) -> None',
	argNames = ('target', 'level', 'xoffset', 'width', 'format', 'type', 'pixels',),
)

glTexSubImage2DEXT = platform.createExtensionFunction( 
	'glTexSubImage2DEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint, constants.GLint, constants.GLint, constants.GLsizei, constants.GLsizei, constants.GLenum, constants.GLenum, ctypes.c_void_p,),
	doc = 'glTexSubImage2DEXT( GLenum(target), GLint(level), GLint(xoffset), GLint(yoffset), GLsizei(width), GLsizei(height), GLenum(format), GLenum(type), c_void_p(pixels) ) -> None',
	argNames = ('target', 'level', 'xoffset', 'yoffset', 'width', 'height', 'format', 'type', 'pixels',),
)


def glInitSubtextureEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Helper functions for wrapping array-using operations

These are functions intended to be used in wrapping
GL functions that deal with OpenGL array data-types.
"""
from OpenGL import contextdata, error, wrapper, constants, converters
from OpenGL.arrays import arraydatatype

def asArrayType( typ ):
	"""Create PyConverter to get first argument as array of type"""
	return converters.CallFuncPyConverter( typ.asArray )

def typedPointer( typ, ctyp ):
	"""Create a CResolver to get argument as a pointer to the ctypes type ctyp
	
	i.e. typedPointer( arrays.UIntArray, ctypes.c_uint )
	"""
	import ctypes
	ctyp = ctypes.POINTER( ctyp )
	def typedPointerConverter(  value ):
		"""Get the arg as an array of the appropriate type"""
		return ctypes.cast( typ.dataPointer(value), ctyp)
	return typedPointerConverter
	
def asArrayTypeSize( typ, size ):
	"""Create PyConverter function to get array as type and check size
	
	Produces a raw function, not a PyConverter instance
	"""
	asArray = typ.asArray
	arraySize = typ.arraySize
	def asArraySize( incoming, function, args ):
		result = asArray( incoming )
		actualSize = arraySize(result)
		if actualSize != size:
			raise ValueError(
				"""Expected %r item array, got %r item array"""%(
					size,
					actualSize,
				),
				incoming,
			)
		return result
	return asArraySize

def asVoidArray( ):
	"""Create PyConverter returning incoming as an array of any type"""
	from OpenGL.arrays import ArrayDatatype
	return converters.CallFuncPyConverter( ArrayDatatype.asArray )

class storePointerType( object ):
	"""Store named pointer value in context indexed by constant
	
	pointerName -- named pointer argument 
	constant -- constant used to index in the context storage
	
	Stores the pyArgs (i.e. result of pyConverters) for the named
	pointer argument...
	"""
	def __init__( self, pointerName, constant ):
		self.pointerName = pointerName
		self.constant = constant 
	def finalise( self, wrapper ):
		self.pointerIndex = wrapper.pyArgIndex( self.pointerName )
	def __call__( self, result, baseOperation, pyArgs, cArgs ):
		contextdata.setValue( self.constant, pyArgs[self.pointerIndex] )

def returnPointer( result,baseOperation,pyArgs,cArgs, ):
	"""Return the converted object as result of function
	
	Note: this is a hack that always returns pyArgs[0]!
	"""
	return pyArgs[0]

def setInputArraySizeType( baseOperation, size, type, argName=0 ):
	"""Decorate function with vector-handling code for a single argument
	
	This assumes *no* argument expansion, i.e. a 1:1 correspondence...
	"""
	function = wrapper.wrapper( baseOperation )
	if not hasattr( function, 'returnValues' ):
		function.setReturnValues( returnPointer )
	if size is not None:
		function.setPyConverter( argName, asArrayTypeSize(type, size) )
	else:
		function.setPyConverter( argName, asArrayType(type) )
	function.setCConverter( argName, converters.getPyArgsName( argName ) )
	return function

def arraySizeOfFirstType( typ, default ):
	def arraySizeOfFirst( pyArgs, index, baseOperation ):
		"""Return the array size of the first argument"""
		array = pyArgs[0]
		if array is None:
			return default
		else:
			return typ.unitSize( array )
	return arraySizeOfFirst

class AsArrayOfType( converters.PyConverter ):
	"""Given arrayName and typeName coerce arrayName to array of type typeName"""
	argNames = ( 'arrayName','typeName' )
	indexLookups = ( 
		('arrayIndex', 'arrayName','pyArgIndex'),
		('typeIndex', 'typeName','pyArgIndex'),
	)
	def __init__( self, arrayName='pointer', typeName='type' ):
		self.arrayName = arrayName
		self.typeName = typeName 
	def __call__( self, arg, wrappedOperation, args):
		"""Get the arg as an array of the appropriate type"""
		type = args[ self.typeIndex ]
		arrayType = arraydatatype.GL_CONSTANT_TO_ARRAY_TYPE[ type ]
		return arrayType.asArray( arg )
class AsArrayTyped( converters.PyConverter ):
	"""Given arrayName and arrayType, convert arrayName to array of type"""
	argNames = ( 'arrayName','arrayType' )
	indexLookups = ( 
		('arrayIndex', 'arrayName','pyArgIndex'),
	)
	def __init__( self, arrayName='pointer', arrayType=None ):
		self.arrayName = arrayName
		self.arrayType = arrayType
	def __call__( self, arg, wrappedOperation, args):
		"""Get the arg as an array of the appropriate type"""
		return self.arrayType.asArray( arg )
class AsArrayTypedSize( converters.CConverter ):
	"""Given arrayName and arrayType, determine size of arrayName"""
	argNames = ( 'arrayName','arrayType' )
	indexLookups = ( 
		('arrayIndex', 'arrayName','pyArgIndex'),
	)
	def __init__( self, arrayName='pointer', arrayType=None ):
		self.arrayName = arrayName
		self.arrayType = arrayType
	def __call__( self, pyArgs, index, wrappedOperation ):
		"""Get the arg as an array of the appropriate type"""
		return self.arrayType.arraySize( pyArgs[self.arrayIndex ] )



"""Wrapper/Implementation of the GLU quadrics object for PyOpenGL"""
from OpenGL.raw import GLU as simple
from OpenGL.platform import GLU,createBaseFunction, PLATFORM
import ctypes

class GLUQuadric( simple.GLUquadric ):
	"""Implementation class for GLUQuadric classes in PyOpenGL"""
	FUNCTION_TYPE = PLATFORM.functionTypeFor(PLATFORM.GLU)
	CALLBACK_TYPES = {
		# mapping from "which" GLU enumeration to a ctypes function type
		simple.GLU_ERROR : FUNCTION_TYPE( None, simple.GLenum )
	}
	def addCallback( self, which, function ):
		"""Register a callback for the quadric object
		
		At the moment only GLU_ERROR is supported by OpenGL, but
		we allow for the possibility of more callbacks in the future...
		"""
		callbackType = self.CALLBACK_TYPES.get( which )
		if not callbackType:
			raise ValueError(
				"""Don't have a registered callback type for %r"""%(
					which,
				)
			)
		if not isinstance( function, callbackType ):
			cCallback = callbackType( function )
		else:
			cCallback = function
		GLU.gluQuadricCallback( self, which, cCallback )
		# XXX catch errors!
		if getattr( self, 'callbacks', None ) is None:
			self.callbacks = {}
		self.callbacks[ which ] = cCallback
		return cCallback
GLUquadric = GLUQuadric

def gluQuadricCallback( quadric, which=simple.GLU_ERROR, function=None ):
	"""Set the GLU error callback function"""
	return quadric.addCallback( which, function )

# Override to produce instances of the sub-class...
gluNewQuadric = createBaseFunction( 
	'gluNewQuadric', dll=GLU, resultType=ctypes.POINTER(GLUQuadric), 
	argTypes=[],
	doc="""gluNewQuadric(  ) -> GLUQuadric
	
Create a new GLUQuadric object""", 
	argNames=[],
)

__all__ = (
	'gluNewQuadric',
	'gluQuadricCallback',
	'GLUQuadric',
)



#! /usr/bin/env python
"""Simple GLU Tess-object test w/out combine callback"""
from OpenGLContext import testingcontext
BaseContext, MainFunction = testingcontext.getInteractive()
from OpenGLContext.scenegraph import polygontessellator, vertex
from OpenGL.GL import *
from OpenGL.GLU import *
from Numeric import *


outline = array([
	[ 0, 0, 0],
	[ 1, 1, 0],
	[ -1, 1, 0],
], 'd')

class TestContext( BaseContext ):
	scale = 1.0
	def OnInit( self ):
		self.tess = polygontessellator.PolygonTessellator()
	def Render( self, mode = 0):
		BaseContext.Render( self, mode )
		self.renderCap( self.scale )
	def renderCap( self, scale = 1.0):
		"""The cap is generated with GLU tessellation routines...
		"""
		vertices = [
			vertex.Vertex( (x/scale,y/scale,z/scale) ) for (x,y,z) in outline
		]
		glColor3f( 1,0,0)
		glDisable( GL_LIGHTING) 
		for type, vertices in self.tess.tessellate( vertices, forceTriangles=0 ):
			print ' geometry type %s, %s vertices (GL_TRIANGLES=%s GL_TRIANGLE_FAN=%s GL_TRIANGLE_STRIP=%s)'%(
				type,
				len(vertices),
				GL_TRIANGLES, GL_TRIANGLE_FAN, GL_TRIANGLE_STRIP
			)
			glNormal3f( 0,0,1 )
			glBegin( type )
			for v in vertices:
				glVertex2dv( v.point[:2] )
			glEnd()

if __name__ == "__main__":
	MainFunction ( TestContext)



'''OpenGL extension VERSION.GL_1_4

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/VERSION/GL_1_4.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_VERSION_GL_1_4'
GL_BLEND_DST_RGB = constant.Constant( 'GL_BLEND_DST_RGB', 0x80C8 )
GL_BLEND_SRC_RGB = constant.Constant( 'GL_BLEND_SRC_RGB', 0x80C9 )
GL_BLEND_DST_ALPHA = constant.Constant( 'GL_BLEND_DST_ALPHA', 0x80CA )
GL_BLEND_SRC_ALPHA = constant.Constant( 'GL_BLEND_SRC_ALPHA', 0x80CB )
GL_POINT_SIZE_MIN = constant.Constant( 'GL_POINT_SIZE_MIN', 0x8126 )
GL_POINT_SIZE_MAX = constant.Constant( 'GL_POINT_SIZE_MAX', 0x8127 )
GL_POINT_FADE_THRESHOLD_SIZE = constant.Constant( 'GL_POINT_FADE_THRESHOLD_SIZE', 0x8128 )
GL_POINT_DISTANCE_ATTENUATION = constant.Constant( 'GL_POINT_DISTANCE_ATTENUATION', 0x8129 )
GL_GENERATE_MIPMAP = constant.Constant( 'GL_GENERATE_MIPMAP', 0x8191 )
GL_GENERATE_MIPMAP_HINT = constant.Constant( 'GL_GENERATE_MIPMAP_HINT', 0x8192 )
GL_DEPTH_COMPONENT16 = constant.Constant( 'GL_DEPTH_COMPONENT16', 0x81A5 )
GL_DEPTH_COMPONENT24 = constant.Constant( 'GL_DEPTH_COMPONENT24', 0x81A6 )
GL_DEPTH_COMPONENT32 = constant.Constant( 'GL_DEPTH_COMPONENT32', 0x81A7 )
GL_MIRRORED_REPEAT = constant.Constant( 'GL_MIRRORED_REPEAT', 0x8370 )
GL_FOG_COORDINATE_SOURCE = constant.Constant( 'GL_FOG_COORDINATE_SOURCE', 0x8450 )
GL_FOG_COORDINATE = constant.Constant( 'GL_FOG_COORDINATE', 0x8451 )
GL_FRAGMENT_DEPTH = constant.Constant( 'GL_FRAGMENT_DEPTH', 0x8452 )
GL_CURRENT_FOG_COORDINATE = constant.Constant( 'GL_CURRENT_FOG_COORDINATE', 0x8453 )
GL_FOG_COORDINATE_ARRAY_TYPE = constant.Constant( 'GL_FOG_COORDINATE_ARRAY_TYPE', 0x8454 )
GL_FOG_COORDINATE_ARRAY_STRIDE = constant.Constant( 'GL_FOG_COORDINATE_ARRAY_STRIDE', 0x8455 )
GL_FOG_COORDINATE_ARRAY_POINTER = constant.Constant( 'GL_FOG_COORDINATE_ARRAY_POINTER', 0x8456 )
GL_FOG_COORDINATE_ARRAY = constant.Constant( 'GL_FOG_COORDINATE_ARRAY', 0x8457 )
GL_COLOR_SUM = constant.Constant( 'GL_COLOR_SUM', 0x8458 )
GL_CURRENT_SECONDARY_COLOR = constant.Constant( 'GL_CURRENT_SECONDARY_COLOR', 0x8459 )
GL_SECONDARY_COLOR_ARRAY_SIZE = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY_SIZE', 0x845A )
GL_SECONDARY_COLOR_ARRAY_TYPE = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY_TYPE', 0x845B )
GL_SECONDARY_COLOR_ARRAY_STRIDE = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY_STRIDE', 0x845C )
GL_SECONDARY_COLOR_ARRAY_POINTER = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY_POINTER', 0x845D )
GL_SECONDARY_COLOR_ARRAY = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY', 0x845E )
GL_MAX_TEXTURE_LOD_BIAS = constant.Constant( 'GL_MAX_TEXTURE_LOD_BIAS', 0x84FD )
GL_TEXTURE_FILTER_CONTROL = constant.Constant( 'GL_TEXTURE_FILTER_CONTROL', 0x8500 )
GL_TEXTURE_LOD_BIAS = constant.Constant( 'GL_TEXTURE_LOD_BIAS', 0x8501 )
GL_INCR_WRAP = constant.Constant( 'GL_INCR_WRAP', 0x8507 )
GL_DECR_WRAP = constant.Constant( 'GL_DECR_WRAP', 0x8508 )
GL_TEXTURE_DEPTH_SIZE = constant.Constant( 'GL_TEXTURE_DEPTH_SIZE', 0x884A )
GL_DEPTH_TEXTURE_MODE = constant.Constant( 'GL_DEPTH_TEXTURE_MODE', 0x884B )
GL_TEXTURE_COMPARE_MODE = constant.Constant( 'GL_TEXTURE_COMPARE_MODE', 0x884C )
GL_TEXTURE_COMPARE_FUNC = constant.Constant( 'GL_TEXTURE_COMPARE_FUNC', 0x884D )
GL_COMPARE_R_TO_TEXTURE = constant.Constant( 'GL_COMPARE_R_TO_TEXTURE', 0x884E )
glBlendFuncSeparate = platform.createExtensionFunction( 
	'glBlendFuncSeparate', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLenum, constants.GLenum,),
	doc = 'glBlendFuncSeparate( GLenum(sfactorRGB), GLenum(dfactorRGB), GLenum(sfactorAlpha), GLenum(dfactorAlpha) ) -> None',
	argNames = ('sfactorRGB', 'dfactorRGB', 'sfactorAlpha', 'dfactorAlpha',),
)

glFogCoordf = platform.createExtensionFunction( 
	'glFogCoordf', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLfloat,),
	doc = 'glFogCoordf( GLfloat(coord) ) -> None',
	argNames = ('coord',),
)

glFogCoordfv = platform.createExtensionFunction( 
	'glFogCoordfv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLfloatArray,),
	doc = 'glFogCoordfv( GLfloatArray(coord) ) -> None',
	argNames = ('coord',),
)

glFogCoordd = platform.createExtensionFunction( 
	'glFogCoordd', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLdouble,),
	doc = 'glFogCoordd( GLdouble(coord) ) -> None',
	argNames = ('coord',),
)

glFogCoorddv = platform.createExtensionFunction( 
	'glFogCoorddv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLdoubleArray,),
	doc = 'glFogCoorddv( GLdoubleArray(coord) ) -> None',
	argNames = ('coord',),
)

glFogCoordPointer = platform.createExtensionFunction( 
	'glFogCoordPointer', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLsizei, ctypes.c_void_p,),
	doc = 'glFogCoordPointer( GLenum(type), GLsizei(stride), c_void_p(pointer) ) -> None',
	argNames = ('type', 'stride', 'pointer',),
)

glMultiDrawArrays = platform.createExtensionFunction( 
	'glMultiDrawArrays', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLintArray, arrays.GLsizeiArray, constants.GLsizei,),
	doc = 'glMultiDrawArrays( GLenum(mode), GLintArray(first), GLsizeiArray(count), GLsizei(primcount) ) -> None',
	argNames = ('mode', 'first', 'count', 'primcount',),
)

glMultiDrawElements = platform.createExtensionFunction( 
	'glMultiDrawElements', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLsizeiArray, constants.GLenum, ctypes.POINTER(ctypes.c_void_p), constants.GLsizei,),
	doc = 'glMultiDrawElements( GLenum(mode), GLsizeiArray(count), GLenum(type), POINTER(ctypes.c_void_p)(indices), GLsizei(primcount) ) -> None',
	argNames = ('mode', 'count', 'type', 'indices', 'primcount',),
)

glPointParameterf = platform.createExtensionFunction( 
	'glPointParameterf', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLfloat,),
	doc = 'glPointParameterf( GLenum(pname), GLfloat(param) ) -> None',
	argNames = ('pname', 'param',),
)

glPointParameterfv = platform.createExtensionFunction( 
	'glPointParameterfv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glPointParameterfv( GLenum(pname), GLfloatArray(params) ) -> None',
	argNames = ('pname', 'params',),
)

glPointParameteri = platform.createExtensionFunction( 
	'glPointParameteri', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint,),
	doc = 'glPointParameteri( GLenum(pname), GLint(param) ) -> None',
	argNames = ('pname', 'param',),
)

glPointParameteriv = platform.createExtensionFunction( 
	'glPointParameteriv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLintArray,),
	doc = 'glPointParameteriv( GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('pname', 'params',),
)

glSecondaryColor3b = platform.createExtensionFunction( 
	'glSecondaryColor3b', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLbyte, constants.GLbyte, constants.GLbyte,),
	doc = 'glSecondaryColor3b( GLbyte(red), GLbyte(green), GLbyte(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3bv = platform.createExtensionFunction( 
	'glSecondaryColor3bv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLbyteArray,),
	doc = 'glSecondaryColor3bv( GLbyteArray(v) ) -> None',
	argNames = ('v',),
)

glSecondaryColor3d = platform.createExtensionFunction( 
	'glSecondaryColor3d', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLdouble, constants.GLdouble, constants.GLdouble,),
	doc = 'glSecondaryColor3d( GLdouble(red), GLdouble(green), GLdouble(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3dv = platform.createExtensionFunction( 
	'glSecondaryColor3dv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLdoubleArray,),
	doc = 'glSecondaryColor3dv( GLdoubleArray(v) ) -> None',
	argNames = ('v',),
)

glSecondaryColor3f = platform.createExtensionFunction( 
	'glSecondaryColor3f', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glSecondaryColor3f( GLfloat(red), GLfloat(green), GLfloat(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3fv = platform.createExtensionFunction( 
	'glSecondaryColor3fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLfloatArray,),
	doc = 'glSecondaryColor3fv( GLfloatArray(v) ) -> None',
	argNames = ('v',),
)

glSecondaryColor3i = platform.createExtensionFunction( 
	'glSecondaryColor3i', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glSecondaryColor3i( GLint(red), GLint(green), GLint(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3iv = platform.createExtensionFunction( 
	'glSecondaryColor3iv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLintArray,),
	doc = 'glSecondaryColor3iv( GLintArray(v) ) -> None',
	argNames = ('v',),
)

glSecondaryColor3s = platform.createExtensionFunction( 
	'glSecondaryColor3s', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLshort, constants.GLshort, constants.GLshort,),
	doc = 'glSecondaryColor3s( GLshort(red), GLshort(green), GLshort(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3sv = platform.createExtensionFunction( 
	'glSecondaryColor3sv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLshortArray,),
	doc = 'glSecondaryColor3sv( GLshortArray(v) ) -> None',
	argNames = ('v',),
)

glSecondaryColor3ub = platform.createExtensionFunction( 
	'glSecondaryColor3ub', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLubyte, constants.GLubyte, constants.GLubyte,),
	doc = 'glSecondaryColor3ub( GLubyte(red), GLubyte(green), GLubyte(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3ubv = platform.createExtensionFunction( 
	'glSecondaryColor3ubv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLubyteArray,),
	doc = 'glSecondaryColor3ubv( GLubyteArray(v) ) -> None',
	argNames = ('v',),
)

glSecondaryColor3ui = platform.createExtensionFunction( 
	'glSecondaryColor3ui', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glSecondaryColor3ui( GLuint(red), GLuint(green), GLuint(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3uiv = platform.createExtensionFunction( 
	'glSecondaryColor3uiv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLuintArray,),
	doc = 'glSecondaryColor3uiv( GLuintArray(v) ) -> None',
	argNames = ('v',),
)

glSecondaryColor3us = platform.createExtensionFunction( 
	'glSecondaryColor3us', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLushort, constants.GLushort, constants.GLushort,),
	doc = 'glSecondaryColor3us( GLushort(red), GLushort(green), GLushort(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3usv = platform.createExtensionFunction( 
	'glSecondaryColor3usv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLushortArray,),
	doc = 'glSecondaryColor3usv( GLushortArray(v) ) -> None',
	argNames = ('v',),
)

glSecondaryColorPointer = platform.createExtensionFunction( 
	'glSecondaryColorPointer', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLenum, constants.GLsizei, ctypes.c_void_p,),
	doc = 'glSecondaryColorPointer( GLint(size), GLenum(type), GLsizei(stride), c_void_p(pointer) ) -> None',
	argNames = ('size', 'type', 'stride', 'pointer',),
)

glWindowPos2d = platform.createExtensionFunction( 
	'glWindowPos2d', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLdouble, constants.GLdouble,),
	doc = 'glWindowPos2d( GLdouble(x), GLdouble(y) ) -> None',
	argNames = ('x', 'y',),
)

glWindowPos2dv = platform.createExtensionFunction( 
	'glWindowPos2dv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLdoubleArray,),
	doc = 'glWindowPos2dv( GLdoubleArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos2f = platform.createExtensionFunction( 
	'glWindowPos2f', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLfloat, constants.GLfloat,),
	doc = 'glWindowPos2f( GLfloat(x), GLfloat(y) ) -> None',
	argNames = ('x', 'y',),
)

glWindowPos2fv = platform.createExtensionFunction( 
	'glWindowPos2fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLfloatArray,),
	doc = 'glWindowPos2fv( GLfloatArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos2i = platform.createExtensionFunction( 
	'glWindowPos2i', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint,),
	doc = 'glWindowPos2i( GLint(x), GLint(y) ) -> None',
	argNames = ('x', 'y',),
)

glWindowPos2iv = platform.createExtensionFunction( 
	'glWindowPos2iv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLintArray,),
	doc = 'glWindowPos2iv( GLintArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos2s = platform.createExtensionFunction( 
	'glWindowPos2s', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLshort, constants.GLshort,),
	doc = 'glWindowPos2s( GLshort(x), GLshort(y) ) -> None',
	argNames = ('x', 'y',),
)

glWindowPos2sv = platform.createExtensionFunction( 
	'glWindowPos2sv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLshortArray,),
	doc = 'glWindowPos2sv( GLshortArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos3d = platform.createExtensionFunction( 
	'glWindowPos3d', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLdouble, constants.GLdouble, constants.GLdouble,),
	doc = 'glWindowPos3d( GLdouble(x), GLdouble(y), GLdouble(z) ) -> None',
	argNames = ('x', 'y', 'z',),
)

glWindowPos3dv = platform.createExtensionFunction( 
	'glWindowPos3dv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLdoubleArray,),
	doc = 'glWindowPos3dv( GLdoubleArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos3f = platform.createExtensionFunction( 
	'glWindowPos3f', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glWindowPos3f( GLfloat(x), GLfloat(y), GLfloat(z) ) -> None',
	argNames = ('x', 'y', 'z',),
)

glWindowPos3fv = platform.createExtensionFunction( 
	'glWindowPos3fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLfloatArray,),
	doc = 'glWindowPos3fv( GLfloatArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos3i = platform.createExtensionFunction( 
	'glWindowPos3i', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glWindowPos3i( GLint(x), GLint(y), GLint(z) ) -> None',
	argNames = ('x', 'y', 'z',),
)

glWindowPos3iv = platform.createExtensionFunction( 
	'glWindowPos3iv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLintArray,),
	doc = 'glWindowPos3iv( GLintArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos3s = platform.createExtensionFunction( 
	'glWindowPos3s', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLshort, constants.GLshort, constants.GLshort,),
	doc = 'glWindowPos3s( GLshort(x), GLshort(y), GLshort(z) ) -> None',
	argNames = ('x', 'y', 'z',),
)

glWindowPos3sv = platform.createExtensionFunction( 
	'glWindowPos3sv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLshortArray,),
	doc = 'glWindowPos3sv( GLshortArray(v) ) -> None',
	argNames = ('v',),
)




"""FreeGLUT extensions to the GLUT API

This module will provide the FreeGLUT extensions if they are available
from the GLUT module.  Note that any other implementation that also provides
these entry points will also retrieve the entry points with this module.
"""
# flags 'freeglut_ext.xml -l /usr/lib64/libglut.so -o freeglut_ext.py -v -kf'
from OpenGL import platform, arrays
from OpenGL.constants import *
from OpenGL import constant
FUNCTION_TYPE = platform.PLATFORM.functionTypeFor( platform.GLUT )

import ctypes
c_int = ctypes.c_int 
c_char_p = ctypes.c_char_p
c_ubyte = ctypes.c_ubyte
c_void_p = ctypes.c_void_p
from OpenGL.GLUT import special

GLUT_ACTION_EXIT = constant.Constant( 'GLUT_ACTION_EXIT', 0 )
GLUT_ACTION_GLUTMAINLOOP_RETURNS = constant.Constant( 'GLUT_ACTION_GLUTMAINLOOP_RETURNS', 1 )
GLUT_ACTION_CONTINUE_EXECUTION = constant.Constant( 'GLUT_ACTION_CONTINUE_EXECUTION', 2 )

GLUT_CREATE_NEW_CONTEXT = constant.Constant( 'GLUT_CREATE_NEW_CONTEXT', 0 )
GLUT_USE_CURRENT_CONTEXT = constant.Constant( 'GLUT_USE_CURRENT_CONTEXT', 1 )

GLUT_ACTION_ON_WINDOW_CLOSE = constant.Constant( 'GLUT_ACTION_ON_WINDOW_CLOSE', 0x01F9 )
GLUT_WINDOW_BORDER_WIDTH = constant.Constant( 'GLUT_WINDOW_BORDER_WIDTH', 0x01FA )
GLUT_WINDOW_HEADER_HEIGHT = constant.Constant( 'GLUT_USE_CURRENT_CONTEXT', 0x01FB )
#GLUT_VERSION = constant.Constant( 'GLUT_VERSION', 0x01FC )
GLUT_RENDERING_CONTEXT = constant.Constant( 'GLUT_RENDERING_CONTEXT', 0x01FD )

# /usr/include/GL/freeglut_ext.h 63
glutMainLoopEvent = platform.createBaseFunction( 
	'glutMainLoopEvent', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutMainLoopEvent(  ) -> None', 
	argNames=(),
)
# /usr/include/GL/freeglut_ext.h 64
glutLeaveMainLoop = platform.createBaseFunction( 
	'glutLeaveMainLoop', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutLeaveMainLoop(  ) -> None', 
	argNames=(),
)

# /usr/include/GL/freeglut_ext.h 69
##glutMouseWheelFunc = platform.createBaseFunction( 
##	'glutMouseWheelFunc', dll=platform.GLUT, resultType=None, 
##	argTypes=[FUNCTION_TYPE(None, c_int, c_int, c_int, c_int)],
##	doc='glutMouseWheelFunc( FUNCTION_TYPE(None, c_int, c_int, c_int, c_int)(callback) ) -> None', 
##	argNames=('callback',),
##)
glutMouseWheelFunc = special.GLUTCallback(
	'MouseWheel', (c_int, c_int, c_int, c_int,), ('wheel','direction','x','y'),
)


# /usr/include/GL/freeglut_ext.h 70
##glutCloseFunc = platform.createBaseFunction( 
##	'glutCloseFunc', dll=platform.GLUT, resultType=None, 
##	argTypes=[FUNCTION_TYPE(None)],
##	doc='glutCloseFunc( FUNCTION_TYPE(None)(callback) ) -> None', 
##	argNames=('callback',),
##)
glutCloseFunc = special.GLUTCallback(
	'Close', (), (),
)

# /usr/include/GL/freeglut_ext.h 71
##glutWMCloseFunc = platform.createBaseFunction( 
##	'glutWMCloseFunc', dll=platform.GLUT, resultType=None, 
##	argTypes=[FUNCTION_TYPE(None)],
##	doc='glutWMCloseFunc( FUNCTION_TYPE(None)(callback) ) -> None', 
##	argNames=('callback',),
##)
glutWMCloseFunc = special.GLUTCallback(
	'WMClose', (), (),
)

# /usr/include/GL/freeglut_ext.h 73
##glutMenuDestroyFunc = platform.createBaseFunction( 
##	'glutMenuDestroyFunc', dll=platform.GLUT, resultType=None, 
##	argTypes=[FUNCTION_TYPE(None)],
##	doc='glutMenuDestroyFunc( FUNCTION_TYPE(None)(callback) ) -> None', 
##	argNames=('callback',),
##)
glutMenuDestroyFunc = special.GLUTCallback(
	'MenuDestroy', (), (),
)

# /usr/include/GL/freeglut_ext.h 78
glutSetOption = platform.createBaseFunction( 
	'glutSetOption', dll=platform.GLUT, resultType=None, 
	argTypes=[GLenum, c_int],
	doc='glutSetOption( GLenum(option_flag), c_int(value) ) -> None', 
	argNames=('option_flag', 'value'),
)

# /usr/include/GL/freeglut_ext.h 80
glutGetWindowData = platform.createBaseFunction( 
	'glutGetWindowData', dll=platform.GLUT, resultType=c_void_p, 
	argTypes=[],
	doc='glutGetWindowData(  ) -> c_void_p', 
	argNames=(),
)

# /usr/include/GL/freeglut_ext.h 81
glutSetWindowData = platform.createBaseFunction( 
	'glutSetWindowData', dll=platform.GLUT, resultType=None, 
	argTypes=[c_void_p],
	doc='glutSetWindowData( c_void_p(data) ) -> None', 
	argNames=('data',),
)

# /usr/include/GL/freeglut_ext.h 82
glutGetMenuData = platform.createBaseFunction( 
	'glutGetMenuData', dll=platform.GLUT, resultType=c_void_p, 
	argTypes=[],
	doc='glutGetMenuData(  ) -> c_void_p', 
	argNames=(),
)

# /usr/include/GL/freeglut_ext.h 83
glutSetMenuData = platform.createBaseFunction( 
	'glutSetMenuData', dll=platform.GLUT, resultType=None, 
	argTypes=[c_void_p],
	doc='glutSetMenuData( c_void_p(data) ) -> None', 
	argNames=('data',),
)

# /usr/include/GL/freeglut_ext.h 88
glutBitmapHeight = platform.createBaseFunction( 
	'glutBitmapHeight', dll=platform.GLUT, resultType=c_int, 
	argTypes=[c_void_p],
	doc='glutBitmapHeight( c_void_p(font) ) -> c_int', 
	argNames=('font',),
)

# /usr/include/GL/freeglut_ext.h 89
glutStrokeHeight = platform.createBaseFunction( 
	'glutStrokeHeight', dll=platform.GLUT, resultType=GLfloat, 
	argTypes=[c_void_p],
	doc='glutStrokeHeight( c_void_p(font) ) -> GLfloat', 
	argNames=('font',),
)

# /usr/include/GL/freeglut_ext.h 90
glutBitmapString = platform.createBaseFunction( 
	'glutBitmapString', dll=platform.GLUT, resultType=None, 
	argTypes=[c_void_p, c_char_p],
	doc='glutBitmapString( c_void_p(font), POINTER(c_ubyte)(string) ) -> None', 
	argNames=('font', 'string'),
)
##glutBitmapString = platform.createBaseFunction( 
##	'glutBitmapString', dll=platform.GLUT, resultType=None, 
##	argTypes=[c_void_p, arrays.GLubyteArray],
##	doc='glutBitmapString( c_void_p(font), POINTER(c_ubyte)(string) ) -> None', 
##	argNames=('font', 'string'),
##)
##glutBitmapString = arrays.setInputArraySizeType(
##	glutBitmapString,
##	None, 
##	arrays.GLubyteArray, 
##	'string',
##)

# /usr/include/GL/freeglut_ext.h 91
glutStrokeString = platform.createBaseFunction( 
	'glutStrokeString', dll=platform.GLUT, resultType=None, 
	argTypes=[c_void_p, c_char_p],
	doc='glutStrokeString( c_void_p(font), POINTER(c_ubyte)(string) ) -> None', 
	argNames=('font', 'string'),
)
##glutStrokeString = platform.createBaseFunction( 
##	'glutStrokeString', dll=platform.GLUT, resultType=None, 
##	argTypes=[c_void_p, arrays.GLubyteArray],
##	doc='glutStrokeString( c_void_p(font), POINTER(c_ubyte)(string) ) -> None', 
##	argNames=('font', 'string'),
##)
##glutStrokeString = arrays.setInputArraySizeType(
##	glutStrokeString,
##	None, 
##	arrays.GLubyteArray, 
##	'string',
##)

# /usr/include/GL/freeglut_ext.h 96
glutWireRhombicDodecahedron = platform.createBaseFunction( 
	'glutWireRhombicDodecahedron', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutWireRhombicDodecahedron(  ) -> None', 
	argNames=(),
)

# /usr/include/GL/freeglut_ext.h 97
glutSolidRhombicDodecahedron = platform.createBaseFunction( 
	'glutSolidRhombicDodecahedron', dll=platform.GLUT, resultType=None, 
	argTypes=[],
	doc='glutSolidRhombicDodecahedron(  ) -> None', 
	argNames=(),
)

# /usr/include/GL/freeglut_ext.h 98
glutWireSierpinskiSponge = platform.createBaseFunction( 
	'glutWireSierpinskiSponge', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int, arrays.GLdoubleArray, GLdouble],
	doc='glutWireSierpinskiSponge( c_int(num_levels), arrays.GLdoubleArray(offset), GLdouble(scale) ) -> None', 
	argNames=('num_levels', 'offset', 'scale'),
)

glutWireSierpinskiSponge = arrays.setInputArraySizeType(
	glutWireSierpinskiSponge,
	None, # XXX Could not determine size of argument offset for glutWireSierpinskiSponge arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'offset',
)

# /usr/include/GL/freeglut_ext.h 99
glutSolidSierpinskiSponge = platform.createBaseFunction( 
	'glutSolidSierpinskiSponge', dll=platform.GLUT, resultType=None, 
	argTypes=[c_int, arrays.GLdoubleArray, GLdouble],
	doc='glutSolidSierpinskiSponge( c_int(num_levels), arrays.GLdoubleArray(offset), GLdouble(scale) ) -> None', 
	argNames=('num_levels', 'offset', 'scale'),
)

glutSolidSierpinskiSponge = arrays.setInputArraySizeType(
	glutSolidSierpinskiSponge,
	None, # XXX Could not determine size of argument offset for glutSolidSierpinskiSponge arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'offset',
)

# /usr/include/GL/freeglut_ext.h 100
glutWireCylinder = platform.createBaseFunction( 
	'glutWireCylinder', dll=platform.GLUT, resultType=None, 
	argTypes=[GLdouble, GLdouble, GLint, GLint],
	doc='glutWireCylinder( GLdouble(radius), GLdouble(height), GLint(slices), GLint(stacks) ) -> None', 
	argNames=('radius', 'height', 'slices', 'stacks'),
)

# /usr/include/GL/freeglut_ext.h 101
glutSolidCylinder = platform.createBaseFunction( 
	'glutSolidCylinder', dll=platform.GLUT, resultType=None, 
	argTypes=[GLdouble, GLdouble, GLint, GLint],
	doc='glutSolidCylinder( GLdouble(radius), GLdouble(height), GLint(slices), GLint(stacks) ) -> None', 
	argNames=('radius', 'height', 'slices', 'stacks'),
)

# /usr/include/GL/freeglut_ext.h 106
glutGetProcAddress = platform.createBaseFunction( 
	'glutGetProcAddress', dll=platform.GLUT, resultType=c_void_p, 
	argTypes=[c_char_p],
	doc='glutGetProcAddress( STRING(procName) ) -> c_void_p', 
	argNames=('procName',),
)



'''OpenGL extension ARB.framebuffer_object

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/framebuffer_object.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_framebuffer_object'
GL_INVALID_FRAMEBUFFER_OPERATION = constant.Constant( 'GL_INVALID_FRAMEBUFFER_OPERATION', 0x506 )
GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING', 0x8210 )
GL_FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_COMPONENT_TYPE', 0x8211 )
GL_FRAMEBUFFER_ATTACHMENT_RED_SIZE = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_RED_SIZE', 0x8212 )
GL_FRAMEBUFFER_ATTACHMENT_GREEN_SIZE = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_GREEN_SIZE', 0x8213 )
GL_FRAMEBUFFER_ATTACHMENT_BLUE_SIZE = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_BLUE_SIZE', 0x8214 )
GL_FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE', 0x8215 )
GL_FRAMEBUFFER_ATTACHMENT_DEPTH_SIZE = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_DEPTH_SIZE', 0x8216 )
GL_FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE', 0x8217 )
GL_FRAMEBUFFER_DEFAULT = constant.Constant( 'GL_FRAMEBUFFER_DEFAULT', 0x8218 )
GL_FRAMEBUFFER_UNDEFINED = constant.Constant( 'GL_FRAMEBUFFER_UNDEFINED', 0x8219 )
GL_DEPTH_STENCIL_ATTACHMENT = constant.Constant( 'GL_DEPTH_STENCIL_ATTACHMENT', 0x821A )
GL_INDEX = constant.Constant( 'GL_INDEX', 0x8222 )
GL_MAX_RENDERBUFFER_SIZE = constant.Constant( 'GL_MAX_RENDERBUFFER_SIZE', 0x84E8 )
GL_DEPTH_STENCIL = constant.Constant( 'GL_DEPTH_STENCIL', 0x84F9 )
GL_UNSIGNED_INT_24_8 = constant.Constant( 'GL_UNSIGNED_INT_24_8', 0x84FA )
GL_DEPTH24_STENCIL8 = constant.Constant( 'GL_DEPTH24_STENCIL8', 0x88F0 )
GL_TEXTURE_STENCIL_SIZE = constant.Constant( 'GL_TEXTURE_STENCIL_SIZE', 0x88F1 )
GL_FRAMEBUFFER_BINDING = constant.Constant( 'GL_FRAMEBUFFER_BINDING', 0x8CA6 )
GL_RENDERBUFFER_BINDING = constant.Constant( 'GL_RENDERBUFFER_BINDING', 0x8CA7 )
GL_READ_FRAMEBUFFER = constant.Constant( 'GL_READ_FRAMEBUFFER', 0x8CA8 )
GL_DRAW_FRAMEBUFFER = constant.Constant( 'GL_DRAW_FRAMEBUFFER', 0x8CA9 )
GL_READ_FRAMEBUFFER_BINDING = constant.Constant( 'GL_READ_FRAMEBUFFER_BINDING', 0x8CAA )
GL_RENDERBUFFER_SAMPLES = constant.Constant( 'GL_RENDERBUFFER_SAMPLES', 0x8CAB )
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE', 0x8CD0 )
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME', 0x8CD1 )
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL', 0x8CD2 )
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE', 0x8CD3 )
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER', 0x8CD4 )
GL_FRAMEBUFFER_COMPLETE = constant.Constant( 'GL_FRAMEBUFFER_COMPLETE', 0x8CD5 )
GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT = constant.Constant( 'GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT', 0x8CD6 )
GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT = constant.Constant( 'GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT', 0x8CD7 )
GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER = constant.Constant( 'GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER', 0x8CDB )
GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER = constant.Constant( 'GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER', 0x8CDC )
GL_FRAMEBUFFER_UNSUPPORTED = constant.Constant( 'GL_FRAMEBUFFER_UNSUPPORTED', 0x8CDD )
GL_MAX_COLOR_ATTACHMENTS = constant.Constant( 'GL_MAX_COLOR_ATTACHMENTS', 0x8CDF )
GL_COLOR_ATTACHMENT0 = constant.Constant( 'GL_COLOR_ATTACHMENT0', 0x8CE0 )
GL_COLOR_ATTACHMENT1 = constant.Constant( 'GL_COLOR_ATTACHMENT1', 0x8CE1 )
GL_COLOR_ATTACHMENT2 = constant.Constant( 'GL_COLOR_ATTACHMENT2', 0x8CE2 )
GL_COLOR_ATTACHMENT3 = constant.Constant( 'GL_COLOR_ATTACHMENT3', 0x8CE3 )
GL_COLOR_ATTACHMENT4 = constant.Constant( 'GL_COLOR_ATTACHMENT4', 0x8CE4 )
GL_COLOR_ATTACHMENT5 = constant.Constant( 'GL_COLOR_ATTACHMENT5', 0x8CE5 )
GL_COLOR_ATTACHMENT6 = constant.Constant( 'GL_COLOR_ATTACHMENT6', 0x8CE6 )
GL_COLOR_ATTACHMENT7 = constant.Constant( 'GL_COLOR_ATTACHMENT7', 0x8CE7 )
GL_COLOR_ATTACHMENT8 = constant.Constant( 'GL_COLOR_ATTACHMENT8', 0x8CE8 )
GL_COLOR_ATTACHMENT9 = constant.Constant( 'GL_COLOR_ATTACHMENT9', 0x8CE9 )
GL_COLOR_ATTACHMENT10 = constant.Constant( 'GL_COLOR_ATTACHMENT10', 0x8CEA )
GL_COLOR_ATTACHMENT11 = constant.Constant( 'GL_COLOR_ATTACHMENT11', 0x8CEB )
GL_COLOR_ATTACHMENT12 = constant.Constant( 'GL_COLOR_ATTACHMENT12', 0x8CEC )
GL_COLOR_ATTACHMENT13 = constant.Constant( 'GL_COLOR_ATTACHMENT13', 0x8CED )
GL_COLOR_ATTACHMENT14 = constant.Constant( 'GL_COLOR_ATTACHMENT14', 0x8CEE )
GL_COLOR_ATTACHMENT15 = constant.Constant( 'GL_COLOR_ATTACHMENT15', 0x8CEF )
GL_DEPTH_ATTACHMENT = constant.Constant( 'GL_DEPTH_ATTACHMENT', 0x8D00 )
GL_STENCIL_ATTACHMENT = constant.Constant( 'GL_STENCIL_ATTACHMENT', 0x8D20 )
GL_FRAMEBUFFER = constant.Constant( 'GL_FRAMEBUFFER', 0x8D40 )
GL_RENDERBUFFER = constant.Constant( 'GL_RENDERBUFFER', 0x8D41 )
GL_RENDERBUFFER_WIDTH = constant.Constant( 'GL_RENDERBUFFER_WIDTH', 0x8D42 )
GL_RENDERBUFFER_HEIGHT = constant.Constant( 'GL_RENDERBUFFER_HEIGHT', 0x8D43 )
GL_RENDERBUFFER_INTERNAL_FORMAT = constant.Constant( 'GL_RENDERBUFFER_INTERNAL_FORMAT', 0x8D44 )
GL_STENCIL_INDEX1 = constant.Constant( 'GL_STENCIL_INDEX1', 0x8D46 )
GL_STENCIL_INDEX4 = constant.Constant( 'GL_STENCIL_INDEX4', 0x8D47 )
GL_STENCIL_INDEX8 = constant.Constant( 'GL_STENCIL_INDEX8', 0x8D48 )
GL_STENCIL_INDEX16 = constant.Constant( 'GL_STENCIL_INDEX16', 0x8D49 )
GL_RENDERBUFFER_RED_SIZE = constant.Constant( 'GL_RENDERBUFFER_RED_SIZE', 0x8D50 )
GL_RENDERBUFFER_GREEN_SIZE = constant.Constant( 'GL_RENDERBUFFER_GREEN_SIZE', 0x8D51 )
GL_RENDERBUFFER_BLUE_SIZE = constant.Constant( 'GL_RENDERBUFFER_BLUE_SIZE', 0x8D52 )
GL_RENDERBUFFER_ALPHA_SIZE = constant.Constant( 'GL_RENDERBUFFER_ALPHA_SIZE', 0x8D53 )
GL_RENDERBUFFER_DEPTH_SIZE = constant.Constant( 'GL_RENDERBUFFER_DEPTH_SIZE', 0x8D54 )
GL_RENDERBUFFER_STENCIL_SIZE = constant.Constant( 'GL_RENDERBUFFER_STENCIL_SIZE', 0x8D55 )
GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE = constant.Constant( 'GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE', 0x8D56 )
GL_MAX_SAMPLES = constant.Constant( 'GL_MAX_SAMPLES', 0x8D57 )
glIsRenderbuffer = platform.createExtensionFunction( 
	'glIsRenderbuffer', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLboolean, 
	argTypes=(constants.GLuint,),
	doc = 'glIsRenderbuffer( GLuint(renderbuffer) ) -> constants.GLboolean',
	argNames = ('renderbuffer',),
)

glBindRenderbuffer = platform.createExtensionFunction( 
	'glBindRenderbuffer', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint,),
	doc = 'glBindRenderbuffer( GLenum(target), GLuint(renderbuffer) ) -> None',
	argNames = ('target', 'renderbuffer',),
)

glDeleteRenderbuffers = platform.createExtensionFunction( 
	'glDeleteRenderbuffers', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray,),
	doc = 'glDeleteRenderbuffers( GLsizei(n), GLuintArray(renderbuffers) ) -> None',
	argNames = ('n', 'renderbuffers',),
)

glGenRenderbuffers = platform.createExtensionFunction( 
	'glGenRenderbuffers', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray,),
	doc = 'glGenRenderbuffers( GLsizei(n), GLuintArray(renderbuffers) ) -> None',
	argNames = ('n', 'renderbuffers',),
)

glRenderbufferStorage = platform.createExtensionFunction( 
	'glRenderbufferStorage', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLsizei, constants.GLsizei,),
	doc = 'glRenderbufferStorage( GLenum(target), GLenum(internalformat), GLsizei(width), GLsizei(height) ) -> None',
	argNames = ('target', 'internalformat', 'width', 'height',),
)

glGetRenderbufferParameteriv = platform.createExtensionFunction( 
	'glGetRenderbufferParameteriv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetRenderbufferParameteriv( GLenum(target), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('target', 'pname', 'params',),
)

glIsFramebuffer = platform.createExtensionFunction( 
	'glIsFramebuffer', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLboolean, 
	argTypes=(constants.GLuint,),
	doc = 'glIsFramebuffer( GLuint(framebuffer) ) -> constants.GLboolean',
	argNames = ('framebuffer',),
)

glBindFramebuffer = platform.createExtensionFunction( 
	'glBindFramebuffer', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint,),
	doc = 'glBindFramebuffer( GLenum(target), GLuint(framebuffer) ) -> None',
	argNames = ('target', 'framebuffer',),
)

glDeleteFramebuffers = platform.createExtensionFunction( 
	'glDeleteFramebuffers', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray,),
	doc = 'glDeleteFramebuffers( GLsizei(n), GLuintArray(framebuffers) ) -> None',
	argNames = ('n', 'framebuffers',),
)

glGenFramebuffers = platform.createExtensionFunction( 
	'glGenFramebuffers', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray,),
	doc = 'glGenFramebuffers( GLsizei(n), GLuintArray(framebuffers) ) -> None',
	argNames = ('n', 'framebuffers',),
)

glCheckFramebufferStatus = platform.createExtensionFunction( 
	'glCheckFramebufferStatus', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLenum, 
	argTypes=(constants.GLenum,),
	doc = 'glCheckFramebufferStatus( GLenum(target) ) -> constants.GLenum',
	argNames = ('target',),
)

glFramebufferTexture1D = platform.createExtensionFunction( 
	'glFramebufferTexture1D', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLenum, constants.GLuint, constants.GLint,),
	doc = 'glFramebufferTexture1D( GLenum(target), GLenum(attachment), GLenum(textarget), GLuint(texture), GLint(level) ) -> None',
	argNames = ('target', 'attachment', 'textarget', 'texture', 'level',),
)

glFramebufferTexture2D = platform.createExtensionFunction( 
	'glFramebufferTexture2D', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLenum, constants.GLuint, constants.GLint,),
	doc = 'glFramebufferTexture2D( GLenum(target), GLenum(attachment), GLenum(textarget), GLuint(texture), GLint(level) ) -> None',
	argNames = ('target', 'attachment', 'textarget', 'texture', 'level',),
)

glFramebufferTexture3D = platform.createExtensionFunction( 
	'glFramebufferTexture3D', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLenum, constants.GLuint, constants.GLint, constants.GLint,),
	doc = 'glFramebufferTexture3D( GLenum(target), GLenum(attachment), GLenum(textarget), GLuint(texture), GLint(level), GLint(zoffset) ) -> None',
	argNames = ('target', 'attachment', 'textarget', 'texture', 'level', 'zoffset',),
)

glFramebufferRenderbuffer = platform.createExtensionFunction( 
	'glFramebufferRenderbuffer', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLenum, constants.GLuint,),
	doc = 'glFramebufferRenderbuffer( GLenum(target), GLenum(attachment), GLenum(renderbuffertarget), GLuint(renderbuffer) ) -> None',
	argNames = ('target', 'attachment', 'renderbuffertarget', 'renderbuffer',),
)

glGetFramebufferAttachmentParameteriv = platform.createExtensionFunction( 
	'glGetFramebufferAttachmentParameteriv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetFramebufferAttachmentParameteriv( GLenum(target), GLenum(attachment), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('target', 'attachment', 'pname', 'params',),
)

glGenerateMipmap = platform.createExtensionFunction( 
	'glGenerateMipmap', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum,),
	doc = 'glGenerateMipmap( GLenum(target) ) -> None',
	argNames = ('target',),
)

glBlitFramebuffer = platform.createExtensionFunction( 
	'glBlitFramebuffer', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint, constants.GLint, constants.GLint, constants.GLint, constants.GLint, constants.GLint, constants.GLbitfield, constants.GLenum,),
	doc = 'glBlitFramebuffer( GLint(srcX0), GLint(srcY0), GLint(srcX1), GLint(srcY1), GLint(dstX0), GLint(dstY0), GLint(dstX1), GLint(dstY1), GLbitfield(mask), GLenum(filter) ) -> None',
	argNames = ('srcX0', 'srcY0', 'srcX1', 'srcY1', 'dstX0', 'dstY0', 'dstX1', 'dstY1', 'mask', 'filter',),
)

glRenderbufferStorageMultisample = platform.createExtensionFunction( 
	'glRenderbufferStorageMultisample', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLsizei, constants.GLenum, constants.GLsizei, constants.GLsizei,),
	doc = 'glRenderbufferStorageMultisample( GLenum(target), GLsizei(samples), GLenum(internalformat), GLsizei(width), GLsizei(height) ) -> None',
	argNames = ('target', 'samples', 'internalformat', 'width', 'height',),
)

glFramebufferTextureLayer = platform.createExtensionFunction( 
	'glFramebufferTextureLayer', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLuint, constants.GLint, constants.GLint,),
	doc = 'glFramebufferTextureLayer( GLenum(target), GLenum(attachment), GLuint(texture), GLint(level), GLint(layer) ) -> None',
	argNames = ('target', 'attachment', 'texture', 'level', 'layer',),
)


def glInitFramebufferObjectARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.multi_draw_arrays

Overview (from the spec)
	
	These functions behave identically to the standard OpenGL 1.1 functions
	glDrawArrays() and glDrawElements() except they handle multiple lists of
	vertices in one call. Their main purpose is to allow one function call
	to render more than one primitive such as triangle strip, triangle fan,
	etc.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/multi_draw_arrays.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_multi_draw_arrays'

glMultiDrawArraysEXT = platform.createExtensionFunction( 
	'glMultiDrawArraysEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLintArray, arrays.GLsizeiArray, constants.GLsizei,),
	doc = 'glMultiDrawArraysEXT( GLenum(mode), GLintArray(first), GLsizeiArray(count), GLsizei(primcount) ) -> None',
	argNames = ('mode', 'first', 'count', 'primcount',),
)

glMultiDrawElementsEXT = platform.createExtensionFunction( 
	'glMultiDrawElementsEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLsizeiArray, constants.GLenum, ctypes.POINTER(ctypes.c_void_p), constants.GLsizei,),
	doc = 'glMultiDrawElementsEXT( GLenum(mode), GLsizeiArray(count), GLenum(type), POINTER(ctypes.c_void_p)(indices), GLsizei(primcount) ) -> None',
	argNames = ('mode', 'count', 'type', 'indices', 'primcount',),
)


def glInitMultiDrawArraysEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.instanced_arrays

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/instanced_arrays.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_instanced_arrays'

glVertexAttribDivisor = platform.createExtensionFunction( 
	'glVertexAttribDivisor', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint,),
	doc = 'glVertexAttribDivisor( GLuint(index), GLuint(divisor) ) -> None',
	argNames = ('index', 'divisor',),
)


def glInitInstancedArraysARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.texture_env_dot3

Overview (from the spec)
	
	Adds new operation to the texture combiner operations.
	
	    DOT3_RGB_ARB                    Arg0 <dotprod> Arg1
	    DOT3_RGBA_ARB                   Arg0 <dotprod> Arg1
	
	where Arg0, Arg1 are specified by <params> parameter of 
	TexEnvf, TexEnvi, TexEnvfv, and TexEnviv when the <pname>
	parameter value is SOURCE0_RGB_ARB and SOURCE1_RGB_ARB.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/texture_env_dot3.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_texture_env_dot3'
GL_DOT3_RGB_ARB = constant.Constant( 'GL_DOT3_RGB_ARB', 0x86AE )
GL_DOT3_RGBA_ARB = constant.Constant( 'GL_DOT3_RGBA_ARB', 0x86AF )


def glInitTextureEnvDot3ARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.coordinate_frame

Overview (from the spec)
	
	This extension allows specifying a per-vertex tangent and binormal
	vector in addition to the normal vector, defining a coordinate frame.
	The coordinate frame is used in additional extensions which also build
	on fragment lighting to achieve bump mapping.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/coordinate_frame.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_coordinate_frame'
GL_TANGENT_ARRAY_EXT = constant.Constant( 'GL_TANGENT_ARRAY_EXT', 0x8439 )
GL_BINORMAL_ARRAY_EXT = constant.Constant( 'GL_BINORMAL_ARRAY_EXT', 0x843A )
GL_CURRENT_TANGENT_EXT = constant.Constant( 'GL_CURRENT_TANGENT_EXT', 0x843B )
glget.addGLGetConstant( GL_CURRENT_TANGENT_EXT, (1,) )
GL_CURRENT_BINORMAL_EXT = constant.Constant( 'GL_CURRENT_BINORMAL_EXT', 0x843C )
glget.addGLGetConstant( GL_CURRENT_BINORMAL_EXT, (1,) )
GL_TANGENT_ARRAY_TYPE_EXT = constant.Constant( 'GL_TANGENT_ARRAY_TYPE_EXT', 0x843E )
glget.addGLGetConstant( GL_TANGENT_ARRAY_TYPE_EXT, (1,) )
GL_TANGENT_ARRAY_STRIDE_EXT = constant.Constant( 'GL_TANGENT_ARRAY_STRIDE_EXT', 0x843F )
glget.addGLGetConstant( GL_TANGENT_ARRAY_STRIDE_EXT, (1,) )
GL_BINORMAL_ARRAY_TYPE_EXT = constant.Constant( 'GL_BINORMAL_ARRAY_TYPE_EXT', 0x8440 )
glget.addGLGetConstant( GL_BINORMAL_ARRAY_TYPE_EXT, (1,) )
GL_BINORMAL_ARRAY_STRIDE_EXT = constant.Constant( 'GL_BINORMAL_ARRAY_STRIDE_EXT', 0x8441 )
glget.addGLGetConstant( GL_BINORMAL_ARRAY_STRIDE_EXT, (1,) )
GL_TANGENT_ARRAY_POINTER_EXT = constant.Constant( 'GL_TANGENT_ARRAY_POINTER_EXT', 0x8442 )
GL_BINORMAL_ARRAY_POINTER_EXT = constant.Constant( 'GL_BINORMAL_ARRAY_POINTER_EXT', 0x8443 )
GL_MAP1_TANGENT_EXT = constant.Constant( 'GL_MAP1_TANGENT_EXT', 0x8444 )
GL_MAP2_TANGENT_EXT = constant.Constant( 'GL_MAP2_TANGENT_EXT', 0x8445 )
GL_MAP1_BINORMAL_EXT = constant.Constant( 'GL_MAP1_BINORMAL_EXT', 0x8446 )
GL_MAP2_BINORMAL_EXT = constant.Constant( 'GL_MAP2_BINORMAL_EXT', 0x8447 )
glTangent3bEXT = platform.createExtensionFunction( 
	'glTangent3bEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLbyte, constants.GLbyte, constants.GLbyte,),
	doc = 'glTangent3bEXT( GLbyte(tx), GLbyte(ty), GLbyte(tz) ) -> None',
	argNames = ('tx', 'ty', 'tz',),
)

glTangent3bvEXT = platform.createExtensionFunction( 
	'glTangent3bvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLbyteArray,),
	doc = 'glTangent3bvEXT( GLbyteArray(v) ) -> None',
	argNames = ('v',),
)

glTangent3dEXT = platform.createExtensionFunction( 
	'glTangent3dEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLdouble, constants.GLdouble, constants.GLdouble,),
	doc = 'glTangent3dEXT( GLdouble(tx), GLdouble(ty), GLdouble(tz) ) -> None',
	argNames = ('tx', 'ty', 'tz',),
)

glTangent3dvEXT = platform.createExtensionFunction( 
	'glTangent3dvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLdoubleArray,),
	doc = 'glTangent3dvEXT( GLdoubleArray(v) ) -> None',
	argNames = ('v',),
)

glTangent3fEXT = platform.createExtensionFunction( 
	'glTangent3fEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glTangent3fEXT( GLfloat(tx), GLfloat(ty), GLfloat(tz) ) -> None',
	argNames = ('tx', 'ty', 'tz',),
)

glTangent3fvEXT = platform.createExtensionFunction( 
	'glTangent3fvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLfloatArray,),
	doc = 'glTangent3fvEXT( GLfloatArray(v) ) -> None',
	argNames = ('v',),
)

glTangent3iEXT = platform.createExtensionFunction( 
	'glTangent3iEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glTangent3iEXT( GLint(tx), GLint(ty), GLint(tz) ) -> None',
	argNames = ('tx', 'ty', 'tz',),
)

glTangent3ivEXT = platform.createExtensionFunction( 
	'glTangent3ivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLintArray,),
	doc = 'glTangent3ivEXT( GLintArray(v) ) -> None',
	argNames = ('v',),
)

glTangent3sEXT = platform.createExtensionFunction( 
	'glTangent3sEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLshort, constants.GLshort, constants.GLshort,),
	doc = 'glTangent3sEXT( GLshort(tx), GLshort(ty), GLshort(tz) ) -> None',
	argNames = ('tx', 'ty', 'tz',),
)

glTangent3svEXT = platform.createExtensionFunction( 
	'glTangent3svEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLshortArray,),
	doc = 'glTangent3svEXT( GLshortArray(v) ) -> None',
	argNames = ('v',),
)

glBinormal3bEXT = platform.createExtensionFunction( 
	'glBinormal3bEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLbyte, constants.GLbyte, constants.GLbyte,),
	doc = 'glBinormal3bEXT( GLbyte(bx), GLbyte(by), GLbyte(bz) ) -> None',
	argNames = ('bx', 'by', 'bz',),
)

glBinormal3bvEXT = platform.createExtensionFunction( 
	'glBinormal3bvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLbyteArray,),
	doc = 'glBinormal3bvEXT( GLbyteArray(v) ) -> None',
	argNames = ('v',),
)

glBinormal3dEXT = platform.createExtensionFunction( 
	'glBinormal3dEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLdouble, constants.GLdouble, constants.GLdouble,),
	doc = 'glBinormal3dEXT( GLdouble(bx), GLdouble(by), GLdouble(bz) ) -> None',
	argNames = ('bx', 'by', 'bz',),
)

glBinormal3dvEXT = platform.createExtensionFunction( 
	'glBinormal3dvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLdoubleArray,),
	doc = 'glBinormal3dvEXT( GLdoubleArray(v) ) -> None',
	argNames = ('v',),
)

glBinormal3fEXT = platform.createExtensionFunction( 
	'glBinormal3fEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glBinormal3fEXT( GLfloat(bx), GLfloat(by), GLfloat(bz) ) -> None',
	argNames = ('bx', 'by', 'bz',),
)

glBinormal3fvEXT = platform.createExtensionFunction( 
	'glBinormal3fvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLfloatArray,),
	doc = 'glBinormal3fvEXT( GLfloatArray(v) ) -> None',
	argNames = ('v',),
)

glBinormal3iEXT = platform.createExtensionFunction( 
	'glBinormal3iEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glBinormal3iEXT( GLint(bx), GLint(by), GLint(bz) ) -> None',
	argNames = ('bx', 'by', 'bz',),
)

glBinormal3ivEXT = platform.createExtensionFunction( 
	'glBinormal3ivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLintArray,),
	doc = 'glBinormal3ivEXT( GLintArray(v) ) -> None',
	argNames = ('v',),
)

glBinormal3sEXT = platform.createExtensionFunction( 
	'glBinormal3sEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLshort, constants.GLshort, constants.GLshort,),
	doc = 'glBinormal3sEXT( GLshort(bx), GLshort(by), GLshort(bz) ) -> None',
	argNames = ('bx', 'by', 'bz',),
)

glBinormal3svEXT = platform.createExtensionFunction( 
	'glBinormal3svEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLshortArray,),
	doc = 'glBinormal3svEXT( GLshortArray(v) ) -> None',
	argNames = ('v',),
)

glTangentPointerEXT = platform.createExtensionFunction( 
	'glTangentPointerEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLsizei, ctypes.c_void_p,),
	doc = 'glTangentPointerEXT( GLenum(type), GLsizei(stride), c_void_p(pointer) ) -> None',
	argNames = ('type', 'stride', 'pointer',),
)

glBinormalPointerEXT = platform.createExtensionFunction( 
	'glBinormalPointerEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLsizei, ctypes.c_void_p,),
	doc = 'glBinormalPointerEXT( GLenum(type), GLsizei(stride), c_void_p(pointer) ) -> None',
	argNames = ('type', 'stride', 'pointer',),
)


def glInitCoordinateFrameEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.tag_sample_buffer

Overview (from the spec)
	
	The extensions defines a special purpose fast multisample clear.
	This clear can be used with some restrictions as a significantly
	faster alternative to Clear(DEPTH_BUFFER_BIT).
	
	The extension is based on the RealityEngine architecture and will
	probably never be supported by any other architecture.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SGIX/tag_sample_buffer.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_tag_sample_buffer'

glTagSampleBufferSGIX = platform.createExtensionFunction( 
	'glTagSampleBufferSGIX', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(),
	doc = 'glTagSampleBufferSGIX(  ) -> None',
	argNames = (),
)


def glInitTagSampleBufferSGIX():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""ctypes-based OpenGL wrapper for Python

This is the PyOpenGL 3.x tree, it attempts to provide
a largely compatible API for code written with the 
PyOpenGL 2.x series using the ctypes foreign function 
interface system.

Configuration Variables:

There are a few configuration variables in this top-level
module.  Applications should be the only code that tweaks 
these variables, mid-level libraries should not take it 
upon themselves to disable/enable features at this level.
The implication there is that your library code should be 
able to work with any of the valid configurations available
with these sets of flags.

	ERROR_CHECKING -- if set to a False value before
		importing any OpenGL.* libraries will completely 
		disable error-checking.  This can dramatically
		improve performance, but makes debugging far 
		harder.
		
		This is intended to be turned off *only* in a 
		production environment where you *know* that 
		your code is entirely free of situations where you 
		use exception-handling to handle error conditions,
		i.e. where you are explicitly checking for errors 
		everywhere they can occur in your code.
		
		Default: True 

	ERROR_LOGGING -- If True, then wrap array-handler 
		functions with  error-logging operations so that all exceptions 
		will be reported to log objects in OpenGL.logs, note that 
		this means you will get lots of error logging whenever you 
		have code that tests by trying something and catching an 
		error, this is intended to be turned on only during 
		development so that you can see why something is failing.
		
		Errors are normally logged to the OpenGL.errors logger.
		
		Only triggers if ERROR_CHECKING is True
		
		Default: False
		
	ERROR_ON_COPY -- if set to a True value before 
		importing the numpy array-support module, will 
		cause array operations to raise 
		OpenGL.error.CopyError if an array operation 
		would cause a data-copy in order to match
		data-types.
		
		This feature allows for optimisation of your 
		application.  It should only be enabled during 
		testing stages to prevent raising errors on 
		recoverable conditions at run-time.  
		
		Note that this feature only works with Numpy 
		arrays at the moment.
		
		Default: False
	
	FULL_LOGGING -- If True, then wrap functions with 
		logging operations which reports each call along with its 
		arguments to  the OpenGL.calltrace logger at the INFO 
		level.  This is *extremely* slow.  You should *not* enable 
		this in production code! 
		
		You will need to have a  logging configuration (e.g. 
			logging.basicConfig() 
		) call  in your top-level script to see the results of the 
		logging.
		
		Default: False
	
	ALLOW_NUMPY_SCALARS -- if True, we will wrap 
		all GLint/GLfloat calls conversions with wrappers 
		that allow for passing numpy scalar values.
		
		Note that this is experimental, *not* reliable,
		and very slow!
		
		Note that byte/char types are not wrapped.
		
		Default: False
	
	UNSIGNED_BYTE_IMAGES_AS_STRING -- if True, we will return
		GL_UNSIGNED_BYTE image-data as strings, istead of arrays
		for glReadPixels and glGetTexImage
"""
from OpenGL.version import __version__

ERROR_CHECKING = True
ERROR_LOGGING = False
ERROR_ON_COPY = False

FULL_LOGGING = False
ALLOW_NUMPY_SCALARS = False
UNSIGNED_BYTE_IMAGES_AS_STRING = True

# Declarations of plugins provided by PyOpenGL itself
from OpenGL.plugins import PlatformPlugin, FormatHandler
PlatformPlugin( 'nt', 'OpenGL.platform.win32.Win32Platform' )
PlatformPlugin( 'posix ', 'OpenGL.platform.glx.GLXPlatform' )
PlatformPlugin( 'linux2', 'OpenGL.platform.glx.GLXPlatform' )
PlatformPlugin( 'darwin', 'OpenGL.platform.darwin.DarwinPlatform' )

FormatHandler( 'none', 'OpenGL.arrays.nones.NoneHandler' )
FormatHandler( 'str', 'OpenGL.arrays.strings.StringHandler' )
FormatHandler( 'list', 'OpenGL.arrays.lists.ListHandler' )
FormatHandler( 'numbers', 'OpenGL.arrays.numbers.NumberHandler' )
FormatHandler( 'ctypesarray', 'OpenGL.arrays.ctypesarrays.CtypesArrayHandler' )
FormatHandler( 'ctypesparameter', 'OpenGL.arrays.ctypesparameters.CtypesParameterHandler' )
FormatHandler( 'ctypespointer', 'OpenGL.arrays.ctypespointers.CtypesPointerHandler' )
FormatHandler( 'numpy', 'OpenGL.arrays.numpymodule.NumpyHandler', ['numpy.ndarray'] )
#FormatHandler( 'numarray', 'OpenGL.arrays.numarrays.NumarrayHandler' )
#FormatHandler( 'numeric', 'OpenGL.arrays.numeric.NumericHandler', )



"""Base class for GLU callback-caching structures"""
import ctypes
import weakref

class GLUStruct( object ):
	"""Mix-in class for GLU Structures that want to retain references to callbacks
	
	Also provides original-object-return for the "datapointer" style paremters
	
	Each sub-class must override:
		CALLBACK_TYPES -- maps a "which" constant to a function type 
		CALLBACK_FUNCTION_REGISTRARS -- maps a "which" constant to the 
			registration function for functions of that type
		WRAPPER_METHODS -- maps a "which" consant to a method of the structure 
			that produces a callable around the function which takes care of 
			input/output arguments, data conversions, error handling and the 
			like.
	Creates a dictionary member dataPointers if original-object-return is used
	Creates a dictionary member callbacks if callback registration is used
	"""
	def getAsParam( self ):
		"""Gets as a ctypes pointer to the underlying structure"""
		return ctypes.pointer( self )
	_as_parameter_ = property( getAsParam )
	CALLBACK_TYPES = None
	CALLBACK_FUNCTION_REGISTRARS = None
	WRAPPER_METHODS = None
	def noteObject( self, object ):
		"""Note object for later retrieval as a Python object pointer
		
		This is the registration point for "original object return", returns 
		a void pointer to the Python object, though this is, effectively, an 
		opaque value.
		"""
		identity = id(object)
		try:
			self.dataPointers[ identity ] = object
		except AttributeError, err:
			self.dataPointers = { identity: object }
		return identity
	def originalObject( self, voidPointer ):
		"""Given a void-pointer, try to find our original Python object"""
		if isinstance( voidPointer, (int,long)):
			identity = voidPointer
		elif voidPointer is None:
			return None
		else:
			try:
				identity = voidPointer.value 
			except AttributeError, err:
				identity = voidPointer[0]
		try:
			return self.dataPointers[ identity ]
		except (KeyError,AttributeError), err:
			return voidPointer
	def addCallback( self, which, function ):
		"""Register a callback for this structure object"""
		callbackType = self.CALLBACK_TYPES.get( which )
		if not callbackType:
			raise ValueError(
				"""Don't have a registered callback type for %r"""%(
					which,
				)
			)
		wrapperMethod = self.WRAPPER_METHODS.get( which )
		if wrapperMethod is not None:
			function = getattr(self,wrapperMethod)( function )
		cCallback = callbackType( function )
		# XXX this is ugly, query to ctypes list on how to fix it...
		try:
			self.CALLBACK_FUNCTION_REGISTRARS[which]( self, which, cCallback )
		except ctypes.ArgumentError, err:
			err.args += (which,cCallback)
			raise
		#gluTessCallbackBase( self, which, cCallback)
		# XXX catch errors!
		if getattr( self, 'callbacks', None ) is None:
			self.callbacks = {}
		self.callbacks[ which ] = cCallback
		return cCallback
	def ptrAsArray( self, ptr, length, type ):
		"""Copy length values from ptr into new array of given type"""
		result = type.zeros( (length,) )
		for i in range(length):
			result[i] = ptr[i]
		return result



'''OpenGL extension NV.conditional_render

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/conditional_render.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_conditional_render'
GL_QUERY_WAIT_NV = constant.Constant( 'GL_QUERY_WAIT_NV', 0x8E13 )
GL_QUERY_NO_WAIT_NV = constant.Constant( 'GL_QUERY_NO_WAIT_NV', 0x8E14 )
GL_QUERY_BY_REGION_WAIT_NV = constant.Constant( 'GL_QUERY_BY_REGION_WAIT_NV', 0x8E15 )
GL_QUERY_BY_REGION_NO_WAIT_NV = constant.Constant( 'GL_QUERY_BY_REGION_NO_WAIT_NV', 0x8E16 )
glBeginConditionalRenderNV = platform.createExtensionFunction( 
	'glBeginConditionalRenderNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum,),
	doc = 'glBeginConditionalRenderNV( GLuint(id), GLenum(mode) ) -> None',
	argNames = ('id', 'mode',),
)

glEndConditionalRenderNV = platform.createExtensionFunction( 
	'glEndConditionalRenderNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(),
	doc = 'glEndConditionalRenderNV(  ) -> None',
	argNames = (),
)


def glInitConditionalRenderNV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Test simple functions (i.e. no pointers involved)"""
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
import time
start = time.time()

window = None

def drawText( value, x,y,  windowHeight, windowWidth, step = 18 ):
	"""Draw the given text at given 2D position in window
	"""
	glMatrixMode(GL_PROJECTION);
	# For some reason the GL_PROJECTION_MATRIX is overflowing with a single push!
	# glPushMatrix()
	matrix = glGetDouble( GL_PROJECTION_MATRIX )
	
	glLoadIdentity();
	glOrtho(0.0, windowHeight or 32, 0.0, windowWidth or 32, -1.0, 1.0)
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	glRasterPos2i(x, y);
	lines = 0
	for character in value:
		if character == '\n':
			glRasterPos2i(x, y-(lines*18))
		else:
			glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, ord(character));
	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	# For some reason the GL_PROJECTION_MATRIX is overflowing with a single push!
	# glPopMatrix();
	glLoadMatrixd( matrix ) # should have un-decorated alias for this...
	
	glMatrixMode(GL_MODELVIEW);

def display():
	glutSetWindow(window);
	glClearColor (0.0, 0.0, (time.time()%1.0)/1.0, 0.0)
	glClear (GL_COLOR_BUFFER_BIT)
	drawText( 'hello', 20,20, size[0],size[1] )
	#glutBitmapCharacter( GLUT_BITMAP_8_BY_13, ord('a'))
	glutSolidTeapot( .2 )
	glFlush ()
	glutSwapBuffers()

size = (250,250)

def reshape( *args ):
	print 'reshape', args
	global size 
	size = args
	display()

def ontimer( *args ):
	print 'timer', args, '@time', time.time()-start
	glutTimerFunc( 1000, ontimer, 24 )

def idle():
	if time.time()-start < 10:
		glutSetWindow(window)
		glutPostRedisplay()
	else:
		glutDestroyWindow( window )
		print 'window destroyed'
		import sys
		sys.exit( 0 )

def testFunction( name ):
	def onevent( *args ):
		print '%s -> %s'%(name, ", ".join( [str(a) for a in args ]))
	return onevent



if __name__ == "__main__":
	import sys
	newArgv = glutInit(sys.argv)
	print 'newArguments', newArgv
	glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB )
	glutInitWindowSize(250, 250)
	glutInitWindowPosition(100, 100)
	window = glutCreateWindow("hello")
	print 'window', repr(window)
	glutDisplayFunc(display)
	glutReshapeFunc(reshape)
	glutMouseFunc(testFunction( 'Mouse' ))
	glutEntryFunc(testFunction( 'Entry' ))
	glutKeyboardFunc( testFunction( 'Keyboard' ))
	glutKeyboardUpFunc( testFunction( 'KeyboardUp' ))
	glutMotionFunc( testFunction( 'Motion' ))
	glutPassiveMotionFunc( testFunction( 'PassiveMotion' ))
	glutVisibilityFunc( testFunction( 'Visibility' ))
	glutWindowStatusFunc( testFunction( 'WindowStatus' ))
	glutSpecialFunc( testFunction( 'Special' ))
	glutSpecialUpFunc( testFunction( 'SpecialUp' ))
	glutTimerFunc( 1000, ontimer, 23 )
	
	glutIdleFunc( idle )
	glutMainLoop()



'''OpenGL extension NV.texgen_reflection

Overview (from the spec)
	
	This extension provides two new texture coordinate generation modes
	that are useful texture-based lighting and environment mapping.
	The reflection map mode generates texture coordinates (s,t,r)
	matching the vertex's eye-space reflection vector.  The reflection
	map mode is useful for environment mapping without the singularity
	inherent in sphere mapping.  The normal map mode generates texture
	coordinates (s,t,r) matching the vertex's transformed eye-space
	normal.  The normal map mode is useful for sophisticated cube map
	texturing-based diffuse lighting models.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/texgen_reflection.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_texgen_reflection'
GL_NORMAL_MAP_NV = constant.Constant( 'GL_NORMAL_MAP_NV', 0x8511 )
GL_REFLECTION_MAP_NV = constant.Constant( 'GL_REFLECTION_MAP_NV', 0x8512 )


def glInitTexgenReflectionNV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ATI.element_array

Overview (from the spec)
	
	This extension provides a mechanism for an application to create
	an array of index data for use in specifying geometric primitives.
	
	This extension is most useful when used in conjunction with the
	ATI_vertex_array_object extension. ATI_vertex_array_object
	provides an interface for storing vertex array data in persistent,
	hardware-addressable memory. In cases where large amounts of
	vertex data are in use, the index data used to construct
	primitives (typically as passed to the GL through DrawElements)
	can impose a significant bandwidth burden. ATI_element_array
	allows the application to specify independent arrays of elements,
	which can then be cached using ATI_vertex_array_object.
	

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ATI/element_array.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ATI_element_array'
GL_ELEMENT_ARRAY_ATI = constant.Constant( 'GL_ELEMENT_ARRAY_ATI', 0x8768 )
GL_ELEMENT_ARRAY_TYPE_ATI = constant.Constant( 'GL_ELEMENT_ARRAY_TYPE_ATI', 0x8769 )
glget.addGLGetConstant( GL_ELEMENT_ARRAY_TYPE_ATI, (1,) )
GL_ELEMENT_ARRAY_POINTER_ATI = constant.Constant( 'GL_ELEMENT_ARRAY_POINTER_ATI', 0x876A )
glElementPointerATI = platform.createExtensionFunction( 
	'glElementPointerATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, ctypes.c_void_p,),
	doc = 'glElementPointerATI( GLenum(type), c_void_p(pointer) ) -> None',
	argNames = ('type', 'pointer',),
)

glDrawElementArrayATI = platform.createExtensionFunction( 
	'glDrawElementArrayATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLsizei,),
	doc = 'glDrawElementArrayATI( GLenum(mode), GLsizei(count) ) -> None',
	argNames = ('mode', 'count',),
)

glDrawRangeElementArrayATI = platform.createExtensionFunction( 
	'glDrawRangeElementArrayATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint, constants.GLsizei,),
	doc = 'glDrawRangeElementArrayATI( GLenum(mode), GLuint(start), GLuint(end), GLsizei(count) ) -> None',
	argNames = ('mode', 'start', 'end', 'count',),
)


def glInitElementArrayATI():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.fragment_program_shadow

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.fragment_program_shadow to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.ARB.fragment_program_shadow import *
### END AUTOGENERATED SECTION

# This extension is entirely within the fragment program functionality,
# it doesn't affect the function-level operations AFAICS.



'''OpenGL extension EXT.texture_env_add

Overview (from the spec)
	
	New texture environment function ADD is supported with the following 
	equation: 
	                    Cv = Cf + Ct
	
	New function may be specified by calling TexEnv with ADD token.
	

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/texture_env_add.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_texture_env_add'



def glInitTextureEnvAddEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.clipmap

Overview (from the spec)
	
	Mipmaps provide a general but expensive solution when the texture image
	is very large.  This extension defines clipmaps, which occupy a small
	subset of the memory required by equivalent mipmaps, but provide much
	of the mipmap rendering capabilities.  Clipmaps are especially useful
	for rendering terrain.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SGIX/clipmap.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_clipmap'
GL_LINEAR_CLIPMAP_LINEAR_SGIX = constant.Constant( 'GL_LINEAR_CLIPMAP_LINEAR_SGIX', 0x8170 )
GL_TEXTURE_CLIPMAP_CENTER_SGIX = constant.Constant( 'GL_TEXTURE_CLIPMAP_CENTER_SGIX', 0x8171 )
GL_TEXTURE_CLIPMAP_FRAME_SGIX = constant.Constant( 'GL_TEXTURE_CLIPMAP_FRAME_SGIX', 0x8172 )
GL_TEXTURE_CLIPMAP_OFFSET_SGIX = constant.Constant( 'GL_TEXTURE_CLIPMAP_OFFSET_SGIX', 0x8173 )
GL_TEXTURE_CLIPMAP_VIRTUAL_DEPTH_SGIX = constant.Constant( 'GL_TEXTURE_CLIPMAP_VIRTUAL_DEPTH_SGIX', 0x8174 )
GL_TEXTURE_CLIPMAP_LOD_OFFSET_SGIX = constant.Constant( 'GL_TEXTURE_CLIPMAP_LOD_OFFSET_SGIX', 0x8175 )
GL_TEXTURE_CLIPMAP_DEPTH_SGIX = constant.Constant( 'GL_TEXTURE_CLIPMAP_DEPTH_SGIX', 0x8176 )
GL_MAX_CLIPMAP_DEPTH_SGIX = constant.Constant( 'GL_MAX_CLIPMAP_DEPTH_SGIX', 0x8177 )
GL_MAX_CLIPMAP_VIRTUAL_DEPTH_SGIX = constant.Constant( 'GL_MAX_CLIPMAP_VIRTUAL_DEPTH_SGIX', 0x8178 )
GL_NEAREST_CLIPMAP_NEAREST_SGIX = constant.Constant( 'GL_NEAREST_CLIPMAP_NEAREST_SGIX', 0x844D )
GL_NEAREST_CLIPMAP_LINEAR_SGIX = constant.Constant( 'GL_NEAREST_CLIPMAP_LINEAR_SGIX', 0x844E )
GL_LINEAR_CLIPMAP_NEAREST_SGIX = constant.Constant( 'GL_LINEAR_CLIPMAP_NEAREST_SGIX', 0x844F )


def glInitClipmapSGIX():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.geometry_program4

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/geometry_program4.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_geometry_program4'
GL_LINES_ADJACENCY_EXT = constant.Constant( 'GL_LINES_ADJACENCY_EXT', 0xA )
GL_LINE_STRIP_ADJACENCY_EXT = constant.Constant( 'GL_LINE_STRIP_ADJACENCY_EXT', 0xB )
GL_TRIANGLES_ADJACENCY_EXT = constant.Constant( 'GL_TRIANGLES_ADJACENCY_EXT', 0xC )
GL_TRIANGLE_STRIP_ADJACENCY_EXT = constant.Constant( 'GL_TRIANGLE_STRIP_ADJACENCY_EXT', 0xD )
GL_GEOMETRY_PROGRAM_NV = constant.Constant( 'GL_GEOMETRY_PROGRAM_NV', 0x8C26 )
GL_MAX_PROGRAM_OUTPUT_VERTICES_NV = constant.Constant( 'GL_MAX_PROGRAM_OUTPUT_VERTICES_NV', 0x8C27 )
GL_MAX_PROGRAM_TOTAL_OUTPUT_COMPONENTS_NV = constant.Constant( 'GL_MAX_PROGRAM_TOTAL_OUTPUT_COMPONENTS_NV', 0x8C28 )
GL_GEOMETRY_VERTICES_OUT_EXT = constant.Constant( 'GL_GEOMETRY_VERTICES_OUT_EXT', 0x8DDA )
GL_GEOMETRY_INPUT_TYPE_EXT = constant.Constant( 'GL_GEOMETRY_INPUT_TYPE_EXT', 0x8DDB )
GL_GEOMETRY_OUTPUT_TYPE_EXT = constant.Constant( 'GL_GEOMETRY_OUTPUT_TYPE_EXT', 0x8DDC )
GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_EXT = constant.Constant( 'GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS_EXT', 0x8C29 )
GL_FRAMEBUFFER_ATTACHMENT_LAYERED_EXT = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_LAYERED_EXT', 0x8DA7 )
GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_EXT = constant.Constant( 'GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS_EXT', 0x8DA8 )
GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_EXT = constant.Constant( 'GL_FRAMEBUFFER_INCOMPLETE_LAYER_COUNT_EXT', 0x8DA9 )
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT = constant.Constant( 'GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LAYER_EXT', 0x8CD4 )
GL_PROGRAM_POINT_SIZE_EXT = constant.Constant( 'GL_PROGRAM_POINT_SIZE_EXT', 0x8642 )
glProgramVertexLimitNV = platform.createExtensionFunction( 
	'glProgramVertexLimitNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint,),
	doc = 'glProgramVertexLimitNV( GLenum(target), GLint(limit) ) -> None',
	argNames = ('target', 'limit',),
)

glFramebufferTextureEXT = platform.createExtensionFunction( 
	'glFramebufferTextureEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLuint, constants.GLint,),
	doc = 'glFramebufferTextureEXT( GLenum(target), GLenum(attachment), GLuint(texture), GLint(level) ) -> None',
	argNames = ('target', 'attachment', 'texture', 'level',),
)

glFramebufferTextureLayerEXT = platform.createExtensionFunction( 
	'glFramebufferTextureLayerEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLuint, constants.GLint, constants.GLint,),
	doc = 'glFramebufferTextureLayerEXT( GLenum(target), GLenum(attachment), GLuint(texture), GLint(level), GLint(layer) ) -> None',
	argNames = ('target', 'attachment', 'texture', 'level', 'layer',),
)

glFramebufferTextureFaceEXT = platform.createExtensionFunction( 
	'glFramebufferTextureFaceEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLuint, constants.GLint, constants.GLenum,),
	doc = 'glFramebufferTextureFaceEXT( GLenum(target), GLenum(attachment), GLuint(texture), GLint(level), GLenum(face) ) -> None',
	argNames = ('target', 'attachment', 'texture', 'level', 'face',),
)


def glInitGeometryProgram4NV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.pixel_texture

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SGIX/pixel_texture.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_pixel_texture'
GL_PIXEL_TEX_GEN_SGIX = constant.Constant( 'GL_PIXEL_TEX_GEN_SGIX', 0x8139 )
GL_PIXEL_TEX_GEN_MODE_SGIX = constant.Constant( 'GL_PIXEL_TEX_GEN_MODE_SGIX', 0x832B )
glPixelTexGenSGIX = platform.createExtensionFunction( 
	'glPixelTexGenSGIX', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum,),
	doc = 'glPixelTexGenSGIX( GLenum(mode) ) -> None',
	argNames = ('mode',),
)


def glInitPixelTextureSGIX():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Extension module support methods

This module provides the tools required to check whether
an extension is available
"""
VERSION_PREFIX = 'GL_VERSION_GL_'
CURRENT_GL_VERSION = ''
AVAILABLE_GL_EXTENSIONS = []
AVAILABLE_GLU_EXTENSIONS = []

def hasGLExtension( specifier ):
	"""Given a string specifier, check for extension being available"""
	global AVAILABLE_GL_EXTENSIONS, CURRENT_GL_VERSION
	specifier = specifier.replace('.','_')
	if specifier.startswith( VERSION_PREFIX ):
		from OpenGL.GL import glGetString, GL_VERSION
		if not CURRENT_GL_VERSION:
			new = glGetString( GL_VERSION )
			if new:
				CURRENT_GL_VERSION = [
					int(x) for x in new.split(' ',1)[0].split( '.' )
				]
			else:
				return False # not yet loaded/supported
		specifier = [
			int(x) 
			for x in specifier[ len(VERSION_PREFIX):].split('_')
		]
		return specifier <= CURRENT_GL_VERSION
	else:
		from OpenGL.GL import glGetString, GL_EXTENSIONS
		if not AVAILABLE_GL_EXTENSIONS:
			AVAILABLE_GL_EXTENSIONS[:] = glGetString( GL_EXTENSIONS ).split()
		return specifier in AVAILABLE_GL_EXTENSIONS

def hasGLUExtension( specifier ):
	"""Given a string specifier, check for extension being available"""
	from OpenGL.GLU import gluGetString, GLU_EXTENSIONS
	if not AVAILABLE_GLU_EXTENSIONS:
		AVAILABLE_GLU_EXTENSIONS[:] = gluGetString( GLU_EXTENSIONS )
	return specifier.replace('.','_') in AVAILABLE_GLU_EXTENSIONS

class _Alternate( object ):
	resolved = False 
	implementation = None
	def __init__( self, name, *alternates ):
		"""Initialize set of alternative implementations of the same function"""
		self.__name__ = name
		self._alternatives = alternates
	def __nonzero__( self ):
		for alternate in self._alternatives:
			if alternate:
				return True 
		return False
	def __call__( self, *args, **named ):
		"""Call, doing a late lookup and bind to find an implementation"""
		for alternate in self._alternatives:
			if alternate:
				self.__class__.__call__ = alternate.__call__
				return self( *args, **named )
		from OpenGL import error
		raise error.NullFunctionError(
			"""Attempt to call an undefined alterate function (%s), check for bool(%s) before calling"""%(
				', '.join([x.__name__ for x in self._alternatives]),
				self.__name__,
			)
		)
def alternate( name, *functions ):
	"""Construct a callable that functions as the first implementation found of given set of alternatives"""
	return type( name, (_Alternate,), {} )( name, *functions )



'''OpenGL extension NV.texgen_emboss

Overview (from the spec)
	
	This extension provides a new texture coordinate generation mode
	suitable for multitexture-based embossing (or bump mapping) effects.
	
	Given two texture units, this extension generates the texture
	coordinates of a second texture unit (an odd-numbered texture unit)
	as a perturbation of a first texture unit (an even-numbered texture
	unit one less than the second texture unit).  The perturbation is
	based on the normal, tangent, and light vectors.  The normal vector
	is supplied by glNormal; the light vector is supplied as a direction
	vector to a specified OpenGL light's position; and the tanget
	vector is supplied by the second texture unit's current texture
	coordinate.  The perturbation is also scaled by program-supplied
	scaling constants.
	
	If both texture units are bound to the same texture representing a
	height field, by subtracting the difference between the resulting two
	filtered texels, programs can achieve a per-pixel embossing effect.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/texgen_emboss.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_texgen_emboss'
GL_EMBOSS_LIGHT_NV = constant.Constant( 'GL_EMBOSS_LIGHT_NV', 0x855D )
GL_EMBOSS_CONSTANT_NV = constant.Constant( 'GL_EMBOSS_CONSTANT_NV', 0x855E )
GL_EMBOSS_MAP_NV = constant.Constant( 'GL_EMBOSS_MAP_NV', 0x855F )


def glInitTexgenEmbossNV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Base class for the various Python data-format storage type APIs

Data-type handlers are specified using OpenGL.plugins module
"""
import ctypes
from OpenGL import plugins

class FormatHandler( object ):
	"""Abstract class describing the handler interface
	
	Each data-type handler is responsible for providing a number of methods
	which allow it to manipulate (and create) instances of the data-type 
	it represents.
	"""
	TYPE_REGISTRY = {}
	LAZY_TYPE_REGISTRY = {}  # more registrations
	HANDLER_REGISTRY = {}
	baseType = None
	typeConstant = None
	HANDLED_TYPES = ()
	preferredOutput = None
	GENERIC_OUTPUT_PREFERENCES = ['numpy','numeric','ctypesarrays']
	ALL_OUTPUT_HANDLERS = []
	def loadAll( cls ):
		"""Load all setuptools-registered FormatHandler classes
		
		register a new datatype with code similar to this in your
		package's setup.py for setuptools:
		
		entry_points = {
			'OpenGL.arrays.formathandler':[
				'numpy = OpenGL.arrays.numpymodule.NumpyHandler',
			],
		}
		"""
		for entrypoint in plugins.FormatHandler.all():
#			if not entrypoint.check:
#				# load automatically...
			cls.loadPlugin( entrypoint )
	@classmethod
	def loadPlugin( cls, entrypoint ):
		"""Load a single entry-point via plugins module"""
		if not entrypoint.loaded:
			try:
				plugin_class = entrypoint.load()
			except ImportError, err:
				from OpenGL import logs
				log = logs.getLog( 'OpenGL.formathandler' )
				log.info(
					'Unable to load registered array format handler %s:\n%s', 
					entrypoint.name, log.getException( err )
				)
			else:
				handler = plugin_class()
				handler.register( handler.HANDLED_TYPES )
				cls.HANDLER_REGISTRY[ entrypoint.name ] = handler
				if hasattr( handler, 'zeros' ):
					cls.ALL_OUTPUT_HANDLERS.append( handler )
			entrypoint.loaded = True
	@classmethod
	def typeLookup( cls, type ):
		"""Lookup handler by data-type"""
		try:
			return cls.TYPE_REGISTRY[ type ]
		except KeyError, err:
			key = '%s.%s'%(type.__module__,type.__name__)
			plugin = cls.LAZY_TYPE_REGISTRY.get( key )
			if plugin:
				cls.loadPlugin( plugin )
				return cls.TYPE_REGISTRY[ type ]
			raise KeyError( """Unable to find data-format handler for %s"""%( type,))
	loadAll = classmethod( loadAll )
	def chooseOutput( cls, preferred=None ):
		"""Choose our output-capable plugin"""
		if not cls.TYPE_REGISTRY:
			cls.loadAll()
		if preferred is not None:
			cls.preferredOutput = preferred
		handler = None
		handler = cls.HANDLER_REGISTRY.get( cls.preferredOutput )
		if not handler:
			for preferred in cls.GENERIC_OUTPUT_PREFERENCES:
				handler = cls.HANDLER_REGISTRY.get( preferred )
				if handler:
					break
		if not handler:
			# look for anything that can do output...
			for handler in cls.ALL_OUTPUT_HANDLERS:
				break
		if not handler:
			raise RuntimeError(
				"""Unable to find any output handler at all (not even ctypes/numpy ones!)"""
			)
		handler.registerReturn()
		return handler
	chooseOutput = classmethod( chooseOutput )
	
	def register( self, types=None ):
		"""Register this class as handler for given set of types"""
		if not isinstance( types, (list,tuple)):
			types = [ types ]
		for type in types:
			self.TYPE_REGISTRY[ type ] = self
	def registerReturn( self ):
		"""Register this handler as the default return-type handler"""
		FormatHandler.RETURN_HANDLER = self
	def registerEquivalent( self, typ, base ):
		"""Register a sub-class for handling as the base-type"""
		
	def from_param( self, value ):
		"""Convert to a ctypes pointer value"""
	def dataPointer( self, value ):
		"""return long for pointer value"""
	def asArray( self, value, typeCode=None ):
		"""Given a value, convert to array representation"""
	def arrayToGLType( self, value ):
		"""Given a value, guess OpenGL type of the corresponding pointer"""
	def arraySize( self, value, typeCode = None ):
		"""Given a data-value, calculate dimensions for the array"""
	def unitSize( self, value, typeCode=None ):
		"""Determine unit size of an array (if possible)"""
		if self.baseType is not None:
			return 
	def dimensions( self, value, typeCode=None ):
		"""Determine dimensions of the passed array value (if possible)"""



"""Numpy (new version) module implementation of the OpenGL-ctypes array interfaces

XXX Need to register handlers for all of the scalar types that numpy returns,
would like to have all return values be int/float if they are of  compatible
type as well.
"""
REGISTRY_NAME = 'numpy'
try:
	import numpy
except ImportError, err:
	raise ImportError( """No numpy module present: %s"""%(err))
import operator
import OpenGL
import ctypes

# numpy's array interface has changed over time :(
testArray = numpy.array( [1,2,3,4],'i' )
if hasattr( testArray, 'ctypes' ):
	if hasattr( testArray.ctypes.data, 'value' ):
		def dataPointer( self, instance ):
			"""Use newer numpy's proper ctypes interface"""
			try:
				return instance.ctypes.data.value
			except AttributeError, err:
				instance = self.asArray( instance )
				return instance.ctypes.data.value
	else:
		def dataPointer( self, instance ):
			"""Use newer numpy's proper ctypes interface"""
			try:
				return instance.ctypes.data
			except AttributeError, err:
				instance = self.asArray( instance )
				return instance.ctypes.data
#	def voidDataPointer( self, instance ):
#		"""Get a void data pointer for the instance"""
#		try:
#			return instance.ctypes.data_as( ctypes.c_void_p )
#		except AttributeError, err:
#			instance = self.asArray( instance )
#			return instance.ctypes.data_as( ctypes.c_void_p )
elif hasattr(testArray,'__array_data__'):
	def dataPointer( self, instance ):
		"""Convert given instance to a data-pointer value (integer)"""
		try:
			return long(instance.__array_data__[0],0)
		except AttributeError, err:
			instance = self.asArray( instance )
			try:
				return long(instance.__array_interface__['data'][0])
			except AttributeError, err:
				return long(instance.__array_data__[0],0)
else:
	def dataPointer( self, instance ):
		"""Convert given instance to a data-pointer value (integer)"""
		try:
			return long(instance.__array_interface__['data'][0])
		except AttributeError, err:
			instance = self.asArray( instance )
			try:
				return long(instance.__array_interface__['data'][0])
			except AttributeError, err:
				return long(instance.__array_data__[0],0)
del testArray

from OpenGL import constants, constant
from OpenGL.arrays import formathandler

class NumpyHandler( formathandler.FormatHandler ):
	"""Numpy-specific data-type handler for OpenGL
	
	Attributes:
	
		ERROR_ON_COPY -- if True, will raise errors 
			if we have to copy an array object in order to produce
			a contiguous array of the correct type.
	"""
	HANDLED_TYPES = (numpy.ndarray, list, tuple )
	dataPointer = dataPointer
	def from_param( self, instance ):
		return ctypes.c_void_p( self.dataPointer( instance ))
	ERROR_ON_COPY = OpenGL.ERROR_ON_COPY
	def zeros( self, dims, typeCode ):
		"""Return Numpy array of zeros in given size"""
		return numpy.zeros( dims, GL_TYPE_TO_ARRAY_MAPPING.get(typeCode) or typeCode )
	def arrayToGLType( self, value ):
		"""Given a value, guess OpenGL type of the corresponding pointer"""
		typeCode = value.dtype.char
		constant = ARRAY_TO_GL_TYPE_MAPPING.get( typeCode )
		if constant is None:
			raise TypeError(
				"""Don't know GL type for array of type %r, known types: %s\nvalue:%s"""%(
					typeCode, ARRAY_TO_GL_TYPE_MAPPING.keys(), value,
				)
			)
		return constant
	def arraySize( self, value, typeCode = None ):
		"""Given a data-value, calculate dimensions for the array"""
		try:
			dimValue = value.shape
		except AttributeError, err:
			# XXX it's a list or a tuple, how do we determine dimensions there???
			# for now we'll just punt and convert to an array first...
			value = self.asArray( value, typeCode )
			dimValue = value.shape 
		dims = 1
		for dim in dimValue:
			dims *= dim 
		return dims 
	def arrayByteCount( self, value, typeCode = None ):
		"""Given a data-value, calculate number of bytes required to represent"""
		try:
			return self.arraySize( value, typeCode ) * value.itemsize
		except AttributeError, err:
			value = self.asArray( value, typeCode )
			return self.arraySize( value, typeCode ) * value.itemsize
	def asArray( self, value, typeCode=None ):
		"""Convert given value to an array value of given typeCode"""
		if value is None:
			return value
		else:
			return self.contiguous( value, typeCode )

	def contiguous( self, source, typeCode=None ):
		"""Get contiguous array from source
		
		source -- numpy Python array (or compatible object)
			for use as the data source.  If this is not a contiguous
			array of the given typeCode, a copy will be made, 
			otherwise will just be returned unchanged.
		typeCode -- optional 1-character typeCode specifier for
			the numpy.array function.
			
		All gl*Pointer calls should use contiguous arrays, as non-
		contiguous arrays will be re-copied on every rendering pass.
		Although this doesn't raise an error, it does tend to slow
		down rendering.
		"""
		typeCode = GL_TYPE_TO_ARRAY_MAPPING.get( typeCode )
		if isinstance( source, numpy.ndarray):
			if source.flags.contiguous and (typeCode is None or typeCode==source.dtype.char):
				return source
			elif (source.flags.contiguous and self.ERROR_ON_COPY):
				from OpenGL import error
				raise error.CopyError(
					"""Array of type %r passed, required array of type %r""",
					source.dtype.char, typeCode,
				)
			else:
				# We have to do astype to avoid errors about unsafe conversions
				# XXX Confirm that this will *always* create a new contiguous array 
				# XXX Guard against wacky conversion types like uint to float, where
				# we really don't want to have the C-level conversion occur.
				# XXX ascontiguousarray is apparently now available in numpy!
				if self.ERROR_ON_COPY:
					from OpenGL import error
					raise error.CopyError(
						"""Non-contiguous array passed""",
						source,
					)
				if typeCode is None:
					typeCode = source.dtype.char
				return numpy.ascontiguousarray( source.astype( typeCode ), typeCode )
		elif typeCode:
			return numpy.ascontiguousarray( source, typeCode )
		else:
			return numpy.ascontiguousarray( source )
	def unitSize( self, value, typeCode=None ):
		"""Determine unit size of an array (if possible)"""
		return value.shape[-1]
	def dimensions( self, value, typeCode=None ):
		"""Determine dimensions of the passed array value (if possible)"""
		return value.shape

try:
	numpy.array( [1], 's' )
	SHORT_TYPE = 's'
except TypeError, err:
	SHORT_TYPE = 'h'
	USHORT_TYPE = 'H'

ARRAY_TO_GL_TYPE_MAPPING = {
	'd': constants.GL_DOUBLE,
	'f': constants.GL_FLOAT,
	'i': constants.GL_INT,
	SHORT_TYPE: constants.GL_SHORT,
	USHORT_TYPE: constants.GL_UNSIGNED_SHORT,
	'B': constants.GL_UNSIGNED_BYTE,
	'c': constants.GL_UNSIGNED_BYTE,
	'b': constants.GL_BYTE,
	'I': constants.GL_UNSIGNED_INT,
}
GL_TYPE_TO_ARRAY_MAPPING = {
	constants.GL_DOUBLE: 'd',
	constants.GL_FLOAT:'f',
	constants.GL_INT: 'i',
	constants.GL_BYTE: 'b',
	constants.GL_SHORT: SHORT_TYPE,
	constants.GL_UNSIGNED_INT: 'I',
	constants.GL_UNSIGNED_BYTE: 'B',
	constants.GL_UNSIGNED_SHORT: USHORT_TYPE,
}



"""The GLUT library implementation via ctypes"""
from OpenGL.raw.GLUT import *

from OpenGL.GLUT.special import *
from OpenGL.GLUT.fonts import *
try:
	from OpenGL.GLUT.freeglut import *
	HAVE_FREEGLUT = False
except ImportError, err:
	HAVE_FREEGLUT = True

